# defivelo-intranet -- Outil métier pour la gestion du Défi Vélo
# Copyright (C) 2016 Didier Raboud <me+defivelo@odyx.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.contrib.auth import get_user_model
from django.contrib.sites.models import Site
from django.core.exceptions import PermissionDenied
from django.db.models import Q
from django.urls import reverse, reverse_lazy
from django.utils import timezone

from rolepermissions.checkers import has_role

from defivelo.roles import has_permission, user_cantons
from defivelo.views import MenuView

from ..forms import SimpleUserProfileForm, UserProfileForm
from ..models import (
    DV_PRIVATE_FIELDS,
    DV_PUBLIC_FIELDS,
    PERSONAL_FIELDS,
    STD_PROFILE_FIELDS,
    USERSTATUS_ARCHIVE,
    USERSTATUS_DELETED,
    UserProfile,
)


class ProfileMixin(MenuView):
    cantons = True
    model = get_user_model()
    context_object_name = "userprofile"
    form_class = UserProfileForm
    profile_fields = STD_PROFILE_FIELDS

    def get_form_class(self):
        try:
            user = self.object
        except AttributeError:
            user = self.get_object()
        if (
            not user
            or user.profile.affiliation_canton
            or UserProfileForm != self.form_class
        ):
            return self.form_class
        else:
            return SimpleUserProfileForm

    def get_context_data(self, **kwargs):
        context = super(ProfileMixin, self).get_context_data(**kwargs)
        context["current_site"] = Site.objects.get_current()
        # Add our menu_category context
        context["menu_category"] = "user"
        context["login_uri"] = self.request.build_absolute_uri(reverse("account_login"))
        return context

    def get_object(self):
        """
        Fallback to myself when accessing the profile
        """
        resolvermatch = self.request.resolver_match
        return self.model.objects.get(
            pk=int(resolvermatch.kwargs.get("pk", self.request.user.pk))
        )

    def get_queryset(self):
        try:
            qs = super(ProfileMixin, self).get_queryset()
        except AttributeError:
            qs = get_user_model().objects
        # Exclude any non-actor or non-formation without the permission
        if not has_permission(self.request.user, "user_view_list_non_collaborator"):
            qs = qs.exclude(
                (Q(profile__formation="") & Q(profile__actor_for__isnull=True))
                | Q(profile__status__in=[USERSTATUS_DELETED, USERSTATUS_ARCHIVE])
            )
        return qs.prefetch_related(
            "groups",
            "profile",
            "profile__actor_for",
            "profile__actor_for__translations",
        ).order_by("first_name", "last_name")

    def get_success_url(self):
        updatepk = self.get_object().pk
        if updatepk == self.request.user.pk:
            return reverse_lazy("profile-detail")
        return reverse_lazy("user-detail", kwargs={"pk": updatepk})

    def get_form(self, *args, **kwargs):
        form = super().get_form(*args, **kwargs)
        disabled_fields = []
        # if the edit user has access, extend the update_profile_fields
        if not has_permission(self.request.user, "user_crud_dv_public_fields"):
            disabled_fields += DV_PUBLIC_FIELDS
        if not has_permission(self.request.user, "user_crud_dv_private_fields"):
            disabled_fields += DV_PRIVATE_FIELDS
        if not has_permission(self.request.user, "user_edit_employee_code"):
            disabled_fields += ("employee_code",)
        for field in form.fields:
            if field in disabled_fields:
                form.fields[field].disabled = True
        return form

    def form_valid(self, form):
        ret = super(ProfileMixin, self).form_valid(form)
        """
        Write the non-model fields
        """
        try:
            user = self.object
        except AttributeError:
            user = self.get_object()

        update_profile_fields = PERSONAL_FIELDS
        # if the edit user has access, extend the update_profile_fields
        if has_permission(self.request.user, "user_crud_dv_public_fields"):
            update_profile_fields += DV_PUBLIC_FIELDS
        if has_permission(self.request.user, "user_crud_dv_private_fields"):
            update_profile_fields += DV_PRIVATE_FIELDS
        if has_permission(self.request.user, "user_edit_employee_code"):
            update_profile_fields += ("employee_code",)

        (userprofile, created) = UserProfile.objects.get_or_create(user=user)
        for field in update_profile_fields:
            if field in form.cleaned_data:
                # For field updates that have date markers, note them properly
                if field in ["status", "bagstatus"]:
                    oldstatus = str(getattr(userprofile, field))
                    newstatus = form.cleaned_data[field]
                    if oldstatus != newstatus:
                        setattr(userprofile, "%s_updatetime" % field, timezone.now())
                modelfield = userprofile._meta.get_field(field)
                if modelfield.many_to_many or modelfield.one_to_many:
                    related_manager = getattr(userprofile, field)
                    related_manager.set(form.cleaned_data[field])
                else:
                    setattr(userprofile, field, form.cleaned_data[field])
        userprofile.save()
        return ret

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        if self.get_form_class() == SimpleUserProfileForm and (
            self.request.user.is_superuser or has_role(self.request.user, "power_user")
        ):
            kwargs["affiliation_canton"] = True

        if self.cantons:
            try:
                kwargs["cantons"] = user_cantons(self.request.user)
            except LookupError:
                pass
        if self.form_class == UserProfileForm:
            # Ne permet qu'au bureau de créer des utilisateurs sans canton d’affiliation
            kwargs["affiliation_canton_required"] = not has_permission(
                self.request.user, "cantons_all"
            )

        return kwargs


class UserSelfAccessMixin(object):
    required_permission = "user_edit_other"

    def dispatch(self, request, *args, **kwargs):
        edit = kwargs.pop("edit", False)
        try:
            usercantons = user_cantons(request.user)
        except LookupError:
            usercantons = []
        user = self.get_object()
        user_cantons_intersection = [
            orga.address_canton
            for orga in user.managed_organizations.all()
            if orga.address_canton in usercantons
        ]
        if (
            # Soit c'est moi
            request.user.pk == user.pk
            or
            # Soit j'ai le droit de lecture/écriture sur tous les cantons
            has_permission(request.user, "cantons_all")
            or
            # Soit j'ai le droit de lecture sur tous les cantons,
            # mais seulement le droit d’écriture sur mes cantons d’affiliation
            (
                has_permission(request.user, self.required_permission)
                and (
                    user_cantons_intersection
                    or
                    # Il est dans mes cantons d’affiliation
                    user.profile.affiliation_canton in usercantons
                    # Je ne fais que le consulter
                    or not edit
                )
            )
        ):
            return super(UserSelfAccessMixin, self).dispatch(request, *args, **kwargs)
        else:
            raise PermissionDenied

    def get_form_kwargs(self):
        kwargs = super(UserSelfAccessMixin, self).get_form_kwargs()
        if has_permission(self.request.user, self.required_permission):
            kwargs["allow_email"] = True
        try:
            kwargs["cantons"] = user_cantons(self.request.user)
        except LookupError:
            pass
        return kwargs



# Generated by Django 1.11.4 on 2017-08-29 12:19
from __future__ import unicode_literals

from django.db import migrations, models

from apps.common import DV_SEASON_LAST_SPRING_MONTH


def fill_in_seasons(apps, schema_editor):
    # We can't import the Person model directly as it may be a newer
    # version than this migration expects. We use the historical version.
    Season = apps.get_model("challenge", "Season")
    for season in Season.objects.all():
        season.year = season.begin.year
        season.season = 1 if season.begin.month <= DV_SEASON_LAST_SPRING_MONTH else 3
        season.save()


class Migration(migrations.Migration):
    dependencies = [
        ("challenge", "0040_auto_20170825_1349"),
    ]

    operations = [
        migrations.AddField(
            model_name="season",
            name="season",
            field=models.PositiveSmallIntegerField(
                choices=[(1, "Printemps"), (3, "Automne")],
                default=1,
                verbose_name="Saison",
            ),
        ),
        migrations.AddField(
            model_name="season",
            name="year",
            field=models.PositiveSmallIntegerField(default=2000, verbose_name="Année"),
            preserve_default=False,
        ),
        migrations.RunPython(fill_in_seasons, migrations.RunPython.noop),
    ]



# Generated by Django 2.2.24 on 2022-01-19 16:55

from django.db import migrations, models

urls = {}


def _save_urls(apps, schema_editor):
    MonthlyCantonalValidationUrl = apps.get_model(
        "salary", "MonthlyCantonalValidationUrl"
    )
    urls.update({obj.id: obj.url for obj in MonthlyCantonalValidationUrl.objects.all()})


def _add_translated_urls(apps, schema_editor):
    MonthlyCantonalValidationUrlTranslation = apps.get_model(
        "salary", "MonthlyCantonalValidationUrlTranslation"
    )
    for master_id, url in urls.items():
        MonthlyCantonalValidationUrlTranslation.objects.filter(
            master_id=master_id
        ).update(url=url)


class Migration(migrations.Migration):
    dependencies = [
        ("salary", "0016_monthlycantonalvalidationurltranslation_label"),
    ]

    operations = [
        migrations.RunPython(_save_urls, migrations.RunPython.noop),
        migrations.RemoveField(
            model_name="monthlycantonalvalidationurl",
            name="url",
        ),
        migrations.AddField(
            model_name="monthlycantonalvalidationurltranslation",
            name="url",
            field=models.URLField(default="", verbose_name="URL"),
            preserve_default=False,
        ),
        migrations.RunPython(_add_translated_urls, migrations.RunPython.noop),
    ]



from django.urls import reverse

from apps.user.tests.factories import UserFactory
from defivelo.tests.utils import PowerUserAuthClient

from ...user import FORMATION_M1, FORMATION_M2
from .. import CHOSEN_AS_HELPER, CHOSEN_AS_LEADER
from ..models import HelperSessionAvailability
from .factories import (
    QualificationActivityFactory,
    QualificationFactory,
)
from .test_seasons import SeasonTestCaseMixin


class QualifsTestCase(SeasonTestCaseMixin):
    """
    Test the qualification views
    """

    def setUp(self):
        self.client = PowerUserAuthClient()
        super().setUp()

    def test_n_helper_validation(self):
        """
        Test that the number of helpers is validated according to the number of M1/M2 configured via n_helpers
        """
        actor = self.client.user
        actor.profile.formation = FORMATION_M1
        actor.profile.save()

        session1 = self.sessions[0]

        # Create some M1 and M2 users and their availability for session1
        m1s = UserFactory.create_batch(3, profile__formation=FORMATION_M1)
        m2s = UserFactory.create_batch(1, profile__formation=FORMATION_M2)
        for m1 in m1s:
            HelperSessionAvailability.objects.get_or_create(
                session=session1,
                helper=m1,
                availability="y",
                chosen_as=CHOSEN_AS_HELPER,
            )
        for m2 in m2s:
            HelperSessionAvailability.objects.get_or_create(
                session=session1,
                helper=m2,
                availability="y",
                chosen_as=CHOSEN_AS_LEADER,
            )

        QualificationActivityFactory()

        q = QualificationFactory.create(session=session1)

        data = {
            "session": session1.pk,
            "name": "Lisa Johnson",
            "class_teacher_fullname": q.class_teacher_fullname,
            "class_teacher_natel": "",
            "n_participants": "14",
            "n_bikes": "6",
            "n_helmets": "",
            "leader": str(m2s[0].pk),
            "helpers": [str(m.pk) for m in m1s],
            "n_helpers": 1,  # too many helpers for the selected n_helpers
            "activity_A": "",
            "activity_B": "",
            "activity_C": "",
            "actor": "",
            "comments": "",
        }

        response = self.client.post(
            reverse(
                "quali-update",
                kwargs={
                    "pk": q.pk,
                    "seasonpk": self.season.pk,
                    "sessionpk": session1.pk,
                },
            ),
            {**data, "n_helpers": "1"},
        )

        self.assertEqual(response.status_code, 200)
        self.assertContains(response, "Pas plus de zéro moniteur·trices 1")

        response = self.client.post(
            reverse(
                "quali-update",
                kwargs={
                    "pk": q.pk,
                    "seasonpk": self.season.pk,
                    "sessionpk": session1.pk,
                },
            ),
            {
                **data,
                "n_helpers": "3",
                "helpers": [str(m.pk) for m in m1s[0:-1]],
            },
        )
        self.assertEqual(response.status_code, 302)



# defivelo-intranet -- Outil métier pour la gestion du Défi Vélo
# Copyright (C) 2015 Didier Raboud <me+defivelo@odyx.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from django.utils.functional import lazy
from django.utils.translation import gettext_lazy as _

from localflavor.ch.ch_states import STATE_CHOICES

STDGLYPHICON = (
    '<span class="glyphicon glyphicon-{icon}" aria-hidden="true"'
    '      title="{title}"></span>'
)

# Cantons où DÉFI VÉLO est actif
DV_STATES = [
    "VD",
    "VS",
    "FR",
    "NE",
    "GE",
    "JU",
    "BS",
    "SG",
    "ZH",
    "BE",
    "LU",
    "AR",
    "SO",
    "AG",
    "BL",
    "GR",
    "SZ",
    "ZG",
    "OW",
    "NW",
]

DV_STATE_COLORS = {
    None: "#000",
    # Dans le même ordre que la charte graphique
    "VD": "#64B984",
    "NE": "#EF7F5E",
    "FR": "#97669E",
    "GE": "#FECC01",
    "JU": "#E84653",
    "BE": "#5EC5F2",
    "VS": "#EF86A1",
    "BS": "#5DBDB2",
    "UR": "#206FB7",
    "SZ": "#E84C0F",
    "OW": "#00595F",
    "NW": "#663265",
    "LU": "#9DC762",
    "AI": "#1D1D1B",
    "AR": "#F7AE1A",
    "SH": "#A4D1A6",
    "BL": "#FEED40",
    "TG": "#363777",
    "SO": "#C61650",
    "ZG": "#00837E",
    "TI": "#B0CB22",
    "GL": "#8075B4",
    "AG": "#8075B4",
    "GR": "#E7206A",
    "SG": "#608A9B",
    "ZH": "#7ABEE4",
    "default": "#56A7DC",
}

DV_STATE_CHOICES = [c for c in STATE_CHOICES if c[0] in DV_STATES]

DV_STATE_CHOICES_WITH_DEFAULT = tuple(
    list(
        (
            (
                "",
                "---------",
            ),
        )
    )
    + list(DV_STATE_CHOICES)
)


def format_with_abbr(label, code):
    if code:
        return f"{label} ({code.upper()})"
    return label


format_with_abbr_lazy = lazy(format_with_abbr, str)

DV_STATE_CHOICES_WITH_ABBR = [
    (v, format_with_abbr_lazy(label, v)) for v, label in DV_STATE_CHOICES
]

DV_LANGUAGES = LANGUAGES = (
    ("fr", _("French")),
    ("de", _("German")),
    ("it", _("Italian")),
    ("en", _("English")),
)

DV_LANGUAGES_WITH_DEFAULT = tuple(
    list(
        (
            (
                "",
                "---------",
            ),
        )
    )
    + list(DV_LANGUAGES)
)

DV_SEASON_SPRING = 1
# DV_SEASON_SUMMER = 2
DV_SEASON_AUTUMN = 3
# DV_SEASON_WINTER = 4

DV_SEASON_CHOICES = (
    (DV_SEASON_SPRING, _("Printemps")),
    (DV_SEASON_AUTUMN, _("Automne")),
)

# Dernier mois des saisons de Printemps
DV_SEASON_LAST_SPRING_MONTH = 7

# État des saisons
DV_SEASON_STATE_PLANNING = 1  # RW for StateManager, -- for helpers
DV_SEASON_STATE_OPEN = 2  # RW for StateManage, RW for helpers
DV_SEASON_STATE_RUNNING = 3  # RW for SM, R- for helpers
DV_SEASON_STATE_FINISHED = 4  # RW for SM, R-- for helpers
DV_SEASON_STATE_ARCHIVED = 5  # R- for SM, R-- for helpers

DV_SEASON_STATES = (
    (DV_SEASON_STATE_PLANNING, _("Planification (invisible)")),
    (DV_SEASON_STATE_OPEN, _("Annoncé (rentrée des disponibilités)")),
    (DV_SEASON_STATE_RUNNING, _("En cours (corrections que par chargé·e·s de projet)")),
    (DV_SEASON_STATE_FINISHED, _("Terminé")),
    (DV_SEASON_STATE_ARCHIVED, _("Archivé")),
)



from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("user", "0017_auto_20160922_1135"),
    ]

    operations = [
        migrations.RenameField(
            model_name="userprofile",
            old_name="activity_cantons",
            new_name="affiliation_canton",
        ),
    ]



# Generated by Django 2.2.16 on 2020-12-09 19:30

import django.core.validators
import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("orga", "0008_auto_20200518_0322"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("challenge", "0075_remove_WS_from_season"),
    ]

    operations = [
        migrations.CreateModel(
            name="Registration",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("date", models.DateField(verbose_name="Date")),
                (
                    "day_time",
                    models.IntegerField(
                        choices=[(0, "Matin"), (1, "Après-midi")], default=0
                    ),
                ),
                (
                    "classes_amount",
                    models.IntegerField(
                        default=1,
                        validators=[
                            django.core.validators.MinValueValidator(1),
                            django.core.validators.MaxValueValidator(6),
                        ],
                        verbose_name="Nb de classes (1 à 6)",
                    ),
                ),
                (
                    "coordinator",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "organization",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="orga.Organization",
                    ),
                ),
            ],
        ),
    ]



# defivelo-intranet -- Outil métier pour la gestion du Défi Vélo
# Copyright (C) 2015 Didier Raboud <me+defivelo@odyx.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.conf import settings
from django.conf.urls.i18n import i18n_patterns
from django.contrib import admin
from django.urls import include, re_path
from django.utils.translation import gettext_lazy as _

from .views.common import HomeView, LicenseView

admin.autodiscover()

urlpatterns = [
    re_path(r"^admin/", include("apps.email_outbox.urls")),
    re_path(r"^admin/", admin.site.urls),
    re_path(r"^i18n/", include("django.conf.urls.i18n")),
    re_path(r"^accounts/", include("allauth.urls")),
    re_path(r"^license/", LicenseView.as_view(), name="license"),
    re_path(r"^agpl-", include("django_agpl.urls")),
    re_path(r"^article/", include("apps.article.urls")),
    re_path(r"^tinymce/", include("tinymce.urls")),
]

urlpatterns += i18n_patterns(
    re_path(r"^$", HomeView.as_view(), name="home"),
    re_path(r"^season/", include("apps.challenge.urls")),
    re_path(r"^orga/", include("apps.orga.urls")),
    re_path(r"^user/", include("apps.user.urls")),
    re_path(r"^info/", include("apps.info.urls")),
    re_path(r"^finance/", include("apps.salary.urls")),
)

if settings.DEBUG:
    import debug_toolbar

    urlpatterns = [
        re_path(r"^__debug__/", include(debug_toolbar.urls)),
    ] + urlpatterns

admin.site.site_header = _("Intranet DÉFI VÉLO")



from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("challenge", "0031_auto_20161013_1600"),
    ]

    operations = [
        migrations.AddField(
            model_name="session",
            name="superleader",
            field=models.ForeignKey(
                blank=True,
                verbose_name="Moniteur +",
                null=True,
                related_name="sess_monplus",
                to=settings.AUTH_USER_MODEL,
                on_delete=models.CASCADE,
            ),
        ),
    ]



# defivelo-intranet -- Outil métier pour la gestion du Défi Vélo
# Copyright (C) 2015 Didier Raboud <me+defivelo@odyx.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.conf import settings
from django.db import models
from django.utils.functional import cached_property
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _
from django.utils.translation import pgettext_lazy as _p

from simple_history.models import HistoricalRecords

from apps.common import STDGLYPHICON
from apps.user import FORMATION_M1, FORMATION_M2, formation_short

from .. import (
    CHOICE_CHOICES,
    CHOSEN_AS_ACTOR,
    CHOSEN_AS_HELPER,
    CHOSEN_AS_LEADER,
    CHOSEN_AS_LEGACY,
    CHOSEN_AS_NOT,
    CHOSEN_AS_REPLACEMENT,
)
from .season import Season
from .session import Session


class HelperSeasonWorkWish(models.Model):
    created_on = models.DateTimeField(auto_now_add=True)
    season = models.ForeignKey(
        Season,
        verbose_name=_p("Singular month", "Mois"),
        related_name="work_wishes",
        on_delete=models.CASCADE,
    )
    helper = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        verbose_name=_("Moniteur"),
        related_name="work_wishes",
        limit_choices_to={"profile__isnull": False},
        on_delete=models.CASCADE,
    )
    amount = models.PositiveSmallIntegerField(
        _("Quantité d’engagements souhaités"), default=0
    )
    history = HistoricalRecords()

    def __str__(self):
        return _("{season}: {helper} aimerait travailler {amount} fois").format(
            session=self.season, helper=self.helper.get_full_name(), amount=self.amount
        )


class HelperSessionAvailability(models.Model):
    AVAILABILITY_CHOICES = (
        ("y", _("Oui")),
        ("i", _("Si nécessaire")),
        ("n", _("Non")),
    )
    created_on = models.DateTimeField(auto_now_add=True)
    session = models.ForeignKey(
        Session,
        verbose_name=_("Session"),
        related_name="availability_statuses",
        on_delete=models.CASCADE,
    )
    helper = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        verbose_name=_("Moniteur"),
        related_name="availabilities",
        limit_choices_to={"profile__isnull": False},
        on_delete=models.CASCADE,
    )
    availability = models.CharField(
        _("Disponible"), max_length=1, choices=AVAILABILITY_CHOICES
    )
    chosen_as = models.PositiveSmallIntegerField(
        _("Sélectionné pour la session comme"),
        choices=CHOICE_CHOICES,
        default=CHOSEN_AS_NOT,
    )
    history = HistoricalRecords()

    @cached_property
    def chosen(self):
        return self.chosen_as != CHOSEN_AS_NOT

    @cached_property
    def chosen_as_icon(self):
        if self.chosen_as == CHOSEN_AS_HELPER:
            # Translators: FORMATION_M1 - Moniteur·trice 1
            return formation_short(FORMATION_M1)
        if self.chosen_as == CHOSEN_AS_LEADER:
            # Translators: FORMATION_M2 - Moniteur·trice 2
            return formation_short(FORMATION_M2)
        if self.chosen_as == CHOSEN_AS_REPLACEMENT:
            # Translators: Moniteur·trice de secours
            return _("S")
        if self.chosen_as == CHOSEN_AS_ACTOR:
            return mark_safe(
                STDGLYPHICON.format(icon="sunglasses", title=_("Intervenant·e"))
            )
        return ""

    @cached_property
    def chosen_as_verb(self):
        if self.chosen_as == CHOSEN_AS_HELPER:
            return _("Moniteur·trice 1")
        if self.chosen_as == CHOSEN_AS_LEADER:
            return _("Moniteur·trice 2")
        if self.chosen_as == CHOSEN_AS_REPLACEMENT:
            return _("Moniteur·trice de secours")
        if self.chosen_as == CHOSEN_AS_ACTOR:
            return _("Intervenant·e")
        if self.chosen_as == CHOSEN_AS_LEGACY:
            return _("Choisi")
        return ""

    @cached_property
    def availability_icon(self):
        title = _("Non")
        icon = "remove-sign"
        if self.availability == "i":
            # If needed
            title = _("Si nécessaire")
            icon = "ok-circle"
        elif self.availability == "y":
            title = _("Oui")
            icon = "ok-sign"
        return mark_safe(STDGLYPHICON.format(icon=icon, title=title))

    class Meta:
        verbose_name = _("Disponibilité par session")
        verbose_name_plural = _("Disponibilités par session")
        ordering = ["session", "helper", "availability"]
        unique_together = (
            (
                "session",
                "helper",
            ),
        )

    def __str__(self):
        is_available = _("n'est pas disponible")
        if self.availability == "y":
            is_available = _("est disponible")
        elif self.availability == "i":
            is_available = _("est disponible si nécessaire")

        return _("{session}: {chosen}{helper} {is_available}").format(
            session=self.session,
            chosen="(%s) " % self.chosen_as if self.chosen else "",
            helper=self.helper.get_full_name(),
            is_available=is_available,
        )



# defivelo-intranet -- Outil métier pour la gestion du Défi Vélo
# Copyright (C) 2016 Didier Raboud <me+defivelo@odyx.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from datetime import date, time, timedelta

from django.urls import reverse

from bs4 import BeautifulSoup

from defivelo.tests.utils import AuthClient, PowerUserAuthClient, StateManagerAuthClient

from ..models import Invoice
from .factories import InvoiceFactory, InvoiceLineFactory
from .test_seasons import SeasonTestCaseMixin

season_urls = ["invoice-orga-list"]
invoice_list_urls = ["invoice-create", "invoice-list"]


def test_invoiceline_out_of_date(db):
    invoiceline = InvoiceLineFactory()
    # Make sure it is not up-to-date
    invoiceline.nb_bikes = invoiceline.session.n_bikes + 1
    assert not invoiceline.is_up_to_date
    invoiceline.refresh()
    assert invoiceline.is_up_to_date


def test_invoiceline_without_session_is_always_out_of_date(db):
    invoiceline = InvoiceLineFactory()
    # Make sure it is not up-to-date by deleting its Session
    invoiceline.session = None
    assert not invoiceline.is_up_to_date
    invoiceline.refresh()
    # It can't become up_to_date
    assert not invoiceline.is_up_to_date


def test_invoice_out_of_date(db):
    invoice = InvoiceFactory()
    # Create 1+2+3+4+5 invoicelines to test all possible sequences
    invoicelines = [InvoiceLineFactory(invoice=invoice) for i in range(0, 15)]

    # Make sure it cannot be up-to-date
    invoicelines[0].nb_bikes = invoicelines[0].session.n_bikes + 1
    assert not invoice.is_up_to_date

    # Now align it all
    invoice.organization.address_canton = invoice.season.cantons[0]
    invoice.organization.save()

    # Create a 1-block, then a space, then a 2-block, then a space, etc
    dayshift = 0
    nth_in_sequence = 1
    sequence_max = 1
    for il in invoicelines:
        il.session.orga = invoice.organization
        # Test the sequences too
        il.session.day = invoice.season.begin + timedelta(days=dayshift)
        if nth_in_sequence < sequence_max:
            nth_in_sequence = nth_in_sequence + 1
        elif nth_in_sequence >= sequence_max:
            nth_in_sequence = 1
            sequence_max = sequence_max + 1
            dayshift = dayshift + 1
        dayshift = dayshift + 1
        il.session.save()
        il.refresh()
        assert il.is_up_to_date
        il.save()
    assert invoice.is_up_to_date

    nth_in_sequence = 1
    sequence_max = 1
    # We have 5 invoicelines, the deduction is 30% for all
    for il in invoicelines:
        if sequence_max == 1:
            assert il.cost_bikes_reduction_percent() == 0
        if sequence_max == 2:
            assert il.cost_bikes_reduction_percent() == 5
        if sequence_max == 3:
            assert il.cost_bikes_reduction_percent() == 10
        if sequence_max == 4:
            assert il.cost_bikes_reduction_percent() == 20
        if sequence_max == 5:
            assert il.cost_bikes_reduction_percent() == 30
        if nth_in_sequence < sequence_max:
            nth_in_sequence = nth_in_sequence + 1
        elif nth_in_sequence >= sequence_max:
            nth_in_sequence = 1
            sequence_max = sequence_max + 1

    # Test that removing any Session will make the Invoice not up_to_date
    invoicelines[0].session.delete()
    assert not invoice.is_up_to_date


def test_bikes_are_accounted_once_per_day(db):
    invoice = InvoiceFactory()
    line_am = InvoiceLineFactory(
        invoice=invoice,
        nb_bikes=10,
        session__day=date(2020, 1, 1),
        session__begin=time(8, 30),
    )
    line_pm = InvoiceLineFactory(
        invoice=invoice,
        nb_bikes=10,
        session__day=date(2020, 1, 1),
        session__begin=time(13, 30),
    )
    assert not line_am.has_cost_bikes and line_am.cost_bikes_reduced == 0
    assert line_pm.has_cost_bikes and line_pm.cost_bikes_reduced > 0

    previous_sum_cost = invoice.sum_cost()
    line_am.cost_bikes = 0
    line_am.save()
    assert invoice.sum_cost() == previous_sum_cost
    line_pm.cost_bikes = 0
    line_pm.save()
    assert invoice.sum_cost() < previous_sum_cost


def test_bikes_are_accounted_for_the_session_with_most_bikes(db):
    invoice = InvoiceFactory()
    line_am = InvoiceLineFactory(
        invoice=invoice,
        nb_bikes=11,
        session__day=date(2020, 1, 1),
        session__begin=time(8, 30),
    )
    line_pm = InvoiceLineFactory(
        invoice=invoice,
        nb_bikes=10,
        session__day=date(2020, 1, 1),
        session__begin=time(13, 30),
    )
    assert line_am.has_cost_bikes and line_am.cost_bikes_reduced > 0
    assert not line_pm.has_cost_bikes and line_pm.cost_bikes_reduced == 0


class InvoiceTestCaseMixin(SeasonTestCaseMixin):
    def setUp(self):
        super().setUp()
        self.invoice = InvoiceFactory(
            season=self.season,
            organization=self.canton_orgas[0],
            status=Invoice.STATUS_DRAFT,
        )
        invoice_kwargs = {
            "seasonpk": self.invoice.season.pk,
            "orgapk": self.invoice.organization.pk,
            "invoiceref": self.invoice.ref,
        }

        self.invoice_update_url = reverse("invoice-update", kwargs=invoice_kwargs)
        self.invoice_detail_url = reverse("invoice-detail", kwargs=invoice_kwargs)


class AuthUserTest(InvoiceTestCaseMixin):
    def setUp(self):
        self.client = AuthClient()
        super().setUp()

    def test_access_to_season_urls(self):
        for symbolicurl in season_urls:
            url = reverse(symbolicurl, kwargs={"seasonpk": self.season.pk})
            response = self.client.get(url, follow=True)
            # No access
            self.assertEqual(response.status_code, 403, url)

    def test_access_to_yearly_urls(self):
        url = reverse("invoices-yearly-list", kwargs={"year": 2020})
        response = self.client.get(url)
        self.assertEqual(response.status_code, 403, url)

    def test_access_to_invoice_lists(self):
        for symbolicurl in invoice_list_urls:
            url = reverse(
                symbolicurl,
                kwargs={"seasonpk": self.season.pk, "orgapk": self.canton_orgas[0].pk},
            )
            response = self.client.get(url, follow=True)
            # No access
            self.assertEqual(response.status_code, 403, url)

    def test_access_to_invoice(self):
        url = reverse(
            "invoice-detail",
            kwargs={
                "seasonpk": self.season.pk,
                "orgapk": self.canton_orgas[0].pk,
                "invoiceref": self.invoice.ref,
            },
        )
        response = self.client.get(url, follow=True)
        # No access
        self.assertEqual(response.status_code, 403, url)


class StateManagerUserTest(InvoiceTestCaseMixin):
    def setUp(self):
        self.client = StateManagerAuthClient()
        super().setUp()

    def test_access_to_season_urls(self):
        for symbolicurl in season_urls:
            url = reverse(symbolicurl, kwargs={"seasonpk": self.season.pk})
            # Final URL is OK
            response = self.client.get(url, follow=True)
            self.assertEqual(response.status_code, 200, url)

    def test_access_to_yearly_urls(self):
        url = reverse("invoices-yearly-list", kwargs={"year": 2020})
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200, url)

    def test_access_to_invoice_lists(self):
        for symbolicurl in invoice_list_urls:
            url = reverse(
                symbolicurl,
                kwargs={"seasonpk": self.season.pk, "orgapk": self.canton_orgas[0].pk},
            )
            response = self.client.get(url, follow=True)
            # Final URL is OK
            self.assertEqual(response.status_code, 200, url)

    def test_access_to_invoice(self):
        url = reverse(
            "invoice-detail",
            kwargs={
                "seasonpk": self.season.pk,
                "orgapk": self.canton_orgas[0].pk,
                "invoiceref": self.invoice.ref,
            },
        )
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200, url)

    def test_invoice_creation(self):
        kwargs = {
            "seasonpk": self.season.pk,
            "orgapk": self.canton_orgas[0].pk,
        }
        url = reverse(
            "invoice-create",
            kwargs=kwargs,
        )
        initial = {"title": "Titolo", "status": Invoice.STATUS_DRAFT}

        response = self.client.post(url, initial)
        i = Invoice.objects.get(**initial)
        kwargs.update({"invoiceref": i.ref})
        self.assertRedirects(
            response,
            reverse("invoice-detail", kwargs=kwargs),
            target_status_code=200,
        )

    def test_invoice_update_title_only(self):
        """
        Updating a title only is OK
        """
        self.assertRedirects(
            self.client.post(
                self.invoice_update_url,
                {"title": "Titre du CdP", "status": self.invoice.status},
            ),
            self.invoice_detail_url,
            target_status_code=200,
        )

        i = Invoice.objects.get(ref=self.invoice.ref)
        self.assertEqual(i.title, "Titre du CdP", i)
        self.assertEqual(i.status, Invoice.STATUS_DRAFT, i)
        self.assertFalse(i.is_locked, i)

    def test_invoice_update_status_locks(self):
        """
        Updating the status away from DRAFT locks
        """
        # Maintenant passe la facture en "Validée"
        self.assertRedirects(
            self.client.post(
                self.invoice_update_url,
                {"title": "Titre 2 du CdP", "status": Invoice.STATUS_VALIDATED},
            ),
            self.invoice_detail_url,
            target_status_code=200,
        )

        i = Invoice.objects.get(ref=self.invoice.ref)
        self.assertEqual(i.title, "Titre 2 du CdP", i)
        self.assertEqual(i.status, Invoice.STATUS_VALIDATED)
        self.assertTrue(i.is_locked, i)

    def test_invoice_update_locked(self):
        """
        Updating a locked invoice fails
        """
        # Passe la facture en "Validée"
        self.invoice.status = Invoice.STATUS_VALIDATED
        self.invoice.save()
        self.assertTrue(self.invoice.is_locked, self.invoice)

        # Lea chargé de projet peut encore l’éditer
        self.assertEqual(self.client.get(self.invoice_update_url).status_code, 200)
        # Mais mettre à jour son statut échoue
        # Enfin. Ça marche, mais rien n’a changé
        self.assertRedirects(
            self.client.post(
                self.invoice_update_url,
                {"title": "Titre 3 du CdP", "status": Invoice.STATUS_DRAFT},
            ),
            self.invoice_detail_url,
            target_status_code=200,
        )

        # Elle n’est bien pas mise à jour
        i = Invoice.objects.get(ref=self.invoice.ref)
        self.assertNotEqual(i.title, "Titre 2 du CdP", i)
        self.assertEqual(i.status, Invoice.STATUS_VALIDATED)

        # Iel peut encore la voir
        self.assertEqual(
            self.client.get(self.invoice_detail_url).status_code,
            200,
        )


class PowerUserTest(InvoiceTestCaseMixin):
    def setUp(self):
        self.client = PowerUserAuthClient()
        super().setUp()

    def test_access_to_season_urls(self):
        for symbolicurl in season_urls:
            url = reverse(symbolicurl, kwargs={"seasonpk": self.season.pk})
            # Final URL is OK
            response = self.client.get(url, follow=True)
            self.assertEqual(response.status_code, 200, url)

    def test_access_to_yearly_urls(self):
        url = reverse("invoices-yearly-list", kwargs={"year": 2020})
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200, url)

    def test_access_to_invoice_lists(self):
        for symbolicurl in invoice_list_urls:
            url = reverse(
                symbolicurl,
                kwargs={"seasonpk": self.season.pk, "orgapk": self.canton_orgas[0].pk},
            )
            response = self.client.get(url, follow=True)
            # Final URL is OK
            self.assertEqual(response.status_code, 200, url)

    def test_access_to_invoice(self):
        url = reverse(
            "invoice-detail",
            kwargs={
                "seasonpk": self.season.pk,
                "orgapk": self.canton_orgas[0].pk,
                "invoiceref": self.invoice.ref,
            },
        )
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200, url)

    def test_invoice_creation(self):
        kwargs = {
            "seasonpk": self.season.pk,
            "orgapk": self.canton_orgas[0].pk,
        }
        url = reverse(
            "invoice-create",
            kwargs=kwargs,
        )
        initial = {"title": "Titolo", "status": Invoice.STATUS_DRAFT}

        response = self.client.post(url, initial)
        i = Invoice.objects.get(**initial)
        kwargs.update({"invoiceref": i.ref})
        self.assertRedirects(
            response,
            reverse("invoice-detail", kwargs=kwargs),
            target_status_code=200,
        )

    def test_invoice_update_title_only(self):
        """
        Updating a title only is OK
        """
        self.assertRedirects(
            self.client.post(
                self.invoice_update_url,
                {"title": "Titre du CdP", "status": self.invoice.status},
            ),
            self.invoice_detail_url,
            target_status_code=200,
        )

        i = Invoice.objects.get(ref=self.invoice.ref)
        self.assertEqual(i.title, "Titre du CdP", i)
        self.assertEqual(i.status, Invoice.STATUS_DRAFT, i)
        self.assertFalse(i.is_locked, i)

    def test_invoice_update_status_locks(self):
        """
        Updating the status away from DRAFT locks
        """
        # Maintenant passe la facture en "Validée"
        self.assertRedirects(
            self.client.post(
                self.invoice_update_url,
                {"title": "Titre 2 du CdP", "status": Invoice.STATUS_VALIDATED},
            ),
            self.invoice_detail_url,
            target_status_code=200,
        )

        i = Invoice.objects.get(ref=self.invoice.ref)
        self.assertEqual(i.title, "Titre 2 du CdP", i)
        self.assertEqual(i.status, Invoice.STATUS_VALIDATED)
        self.assertTrue(i.is_locked, i)

    def test_invoice_update_locked(self):
        """
        Updating a locked invoice fails
        """
        # Passe la facture en "Validée"
        self.invoice.status = Invoice.STATUS_VALIDATED
        self.invoice.save()
        self.assertTrue(self.invoice.is_locked, self.invoice)

        # Lea bureau peut encore l’éditer
        self.assertEqual(self.client.get(self.invoice_update_url).status_code, 200)
        # Et mettre à jour son statut, de retour vers Draft
        self.assertRedirects(
            self.client.post(
                self.invoice_update_url,
                {"title": "Titre 3 du Bureau", "status": Invoice.STATUS_DRAFT},
            ),
            self.invoice_detail_url,
            target_status_code=200,
        )
        # Elle est bien mise à jour
        i = Invoice.objects.get(ref=self.invoice.ref)
        self.assertNotEqual(i.title, "Titre 3 du bureau", i)
        self.assertEqual(i.status, Invoice.STATUS_DRAFT, i)
        self.assertFalse(i.is_locked, i)

        # Iel peut encore la voir
        self.assertEqual(
            self.client.get(self.invoice_detail_url).status_code,
            200,
        )

    def test_draft_invoice_can_print(self):
        url = reverse(
            "invoice-detail",
            kwargs={
                "seasonpk": self.season.pk,
                "orgapk": self.canton_orgas[0].pk,
                "invoiceref": self.invoice.ref,
            },
        )

        for status, status_label in Invoice.STATUS_CHOICES:
            self.invoice.status = status
            self.invoice.save()

            response = self.client.get(url, follow=True)
            self.assertEqual(response.status_code, 200, url)

            watermark = BeautifulSoup(response.content, "html.parser").select_one(
                "#watermark-container"
            )
            self.assertIsNotNone(
                watermark, f"Watermark should be present for status {status}"
            )
            label = watermark.attrs.get("data-label")
            if status == Invoice.STATUS_DRAFT:
                self.assertContains(response, "Imprimer le devis")
                self.assertEqual(
                    label,
                    "Devis",
                    f"Watermark label should be 'Devis' for status {status_label}",
                )
            else:
                self.assertContains(response, "Imprimer")
                self.assertEqual(
                    label,
                    "",
                    f"Watermark label should be empty for status {status_label}",
                )



from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("challenge", "0011_auto_20150903_1658"),
    ]

    operations = [
        migrations.AddField(
            model_name="qualification",
            name="class_teacher_fullname",
            field=models.CharField(
                verbose_name="Enseignant", blank=True, max_length=512
            ),
        ),
        migrations.AddField(
            model_name="qualification",
            name="class_teacher_natel",
            field=models.CharField(
                verbose_name="Natel enseignant", blank=True, max_length=13
            ),
        ),
    ]



# defivelo-intranet -- Outil métier pour la gestion du Défi Vélo
# Copyright (C) 2015 Didier Raboud <me+defivelo@odyx.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.contrib.auth import get_user_model
from django.utils.encoding import force_str
from django.utils.translation import gettext_lazy as _

from import_export import fields, resources, widgets

from defivelo.templatetags.dv_filters import canton_abbr, cantons_abbr

from .models import COLLABORATOR_FIELDS, STD_PROFILE_FIELDS


class MultipleSelectWidget(widgets.Widget):
    def render(self, value, obj=None):
        return ", ".join(value)


class FirstMedWidget(widgets.Widget):
    def render(self, value, object=None):
        final = _("Yes") if value.firstmed_course else _("No")
        if value.firstmed_course_comm:
            final += " - " + value.firstmed_course_comm
        return force_str(final)


class ObjectMethodWidget(widgets.Widget):
    def __init__(self, method, *args, **kwargs):
        self.method = method
        return super(ObjectMethodWidget, self).__init__(*args, **kwargs)

    def render(self, value, object=None):
        attribute = getattr(value, self.method)
        if isinstance(attribute, list):
            attribute = ", ".join(attribute)
        if attribute:
            return force_str(attribute)
        return ""


ALL_PROFILE_FIELDS = tuple(
    ["first_name", "last_name", "email"]
    + [
        "profile__%s" % field
        for field in STD_PROFILE_FIELDS
        if field not in ["firstmed_course_comm"]
    ]
)
ALL_COLLABORATOR_FIELDS = tuple(
    ["first_name", "last_name", "email"]
    + ["profile__%s" % field for field in COLLABORATOR_FIELDS]
)


class UserResource(resources.ModelResource):
    first_name = fields.Field(column_name=_("Prénom"), attribute="first_name")
    last_name = fields.Field(column_name=_("Nom"), attribute="last_name")
    profile__language = fields.Field(
        column_name=_("Langue"), attribute="profile__language"
    )
    profile__languages_challenges = fields.Field(
        column_name=_("Prêt à animer en"),
        attribute="profile__languages_challenges",
        widget=MultipleSelectWidget(),
    )
    email = fields.Field(column_name=_("Email"), attribute="email")
    profile__natel = fields.Field(column_name=_("Natel"), attribute="profile__natel")
    profile__address_street = fields.Field(
        column_name=_("Rue"), attribute="profile__address_street"
    )
    profile__address_no = fields.Field(
        column_name=_("N°"), attribute="profile__address_no"
    )
    profile__address_zip = fields.Field(
        column_name=_("NPA"), attribute="profile__address_zip"
    )
    profile__address_city = fields.Field(
        column_name=_("Ville"), attribute="profile__address_city"
    )
    profile__address_canton = fields.Field(
        column_name=_("Canton de domicile"), attribute="profile__address_canton"
    )
    profile__birthdate = fields.Field(
        column_name=_("Date de naissance"),
        attribute="profile__birthdate",
        widget=widgets.DateWidget(format="%d.%m.%Y"),
    )
    profile__nationality = fields.Field(
        column_name=_("Nationalité"), attribute="profile__nationality"
    )
    profile__work_permit = fields.Field(
        column_name=_("Permis de travail"), attribute="profile__work_permit"
    )
    profile__tax_jurisdiction = fields.Field(
        column_name=_("Lieu d’imposition"), attribute="profile__tax_jurisdiction"
    )
    profile__social_security = fields.Field(
        column_name=_("N° AVS"), attribute="profile__social_security"
    )
    profile__marital_status = fields.Field(
        column_name=_("État civil"),
        attribute="profile",
        widget=ObjectMethodWidget(method="marital_status_full"),
    )
    profile__pedagogical_experience = fields.Field(
        column_name=_("Expérience pédagogique"),
        attribute="profile__pedagogical_experience",
    )
    profile__comments = fields.Field(
        column_name=_("Commentaires"), attribute="profile__comments"
    )
    # From there, specific widgets
    profile__firstmed_course = fields.Field(
        column_name=_("Cours samaritains"), attribute="profile", widget=FirstMedWidget()
    )
    profile__actor_for = fields.Field(
        column_name=_("Intervenant·e"),
        attribute="profile",
        widget=ObjectMethodWidget(method="actor_inline"),
    )
    profile__status = fields.Field(
        column_name=_("Statut"),
        attribute="profile",
        widget=ObjectMethodWidget(method="status_full"),
    )
    profile__bagstatus = fields.Field(
        column_name=_("Sac Défi Vélo"),
        attribute="profile",
        widget=ObjectMethodWidget(method="bagstatus_full"),
    )
    profile__formation = fields.Field(
        column_name=_("Formation"),
        attribute="profile",
        widget=ObjectMethodWidget(method="formation_full"),
    )
    profile__formation_firstdate = fields.Field(
        column_name=_("Date de la première formation"),
        attribute="profile__formation_firstdate",
        widget=widgets.DateWidget(format="%d.%m.%Y"),
    )
    profile__formation_lastdate = fields.Field(
        column_name=_("Date de la dernière formation"),
        attribute="profile__formation_lastdate",
        widget=widgets.DateWidget(format="%d.%m.%Y"),
    )
    profile__affiliation_canton = fields.Field(column_name=_("Canton d’affiliation"))
    profile__activity_cantons = fields.Field(
        column_name=_("Défi Vélo Mobile"),
        attribute="profile__activity_cantons",
        widget=MultipleSelectWidget(),
    )
    profile__employee_code = fields.Field(
        column_name=_("Numéro d’employé Crésus"),
        attribute="profile__employee_code",
    )
    profile__bank_name = fields.Field(
        column_name=_("Nom de la banque"),
        attribute="profile__bank_name",
    )
    profile__iban = fields.Field(
        column_name=_("IBAN"),
        attribute="profile__iban",
    )
    profile__access_level = fields.Field(
        column_name=_("Niveau d’accès"),
        attribute="profile",
        widget=ObjectMethodWidget(method="access_level_text"),
    )
    profile__managed_cantons = fields.Field(
        column_name=_("Cantons gérés"),
        attribute="profile",
        widget=ObjectMethodWidget(method="managed_cantons"),
    )

    profile__updated_at = fields.Field(
        column_name=_("Dernière mise à jour"),
        attribute="profile__updated_at",
        widget=widgets.DateWidget(format="%d.%m.%Y %H:%M"),
    )

    class Meta:
        model = get_user_model()
        fields = ALL_PROFILE_FIELDS
        export_order = ALL_PROFILE_FIELDS

    def dehydrate_profile__address_canton(self, field):
        return canton_abbr(field.profile.address_canton, abbr=False, long=True)

    def dehydrate_profile__affiliation_canton(self, field):
        return canton_abbr(field.profile.affiliation_canton, abbr=False, long=True)

    def dehydrate_profile__activity_cantons(self, field):
        return ", ".join(
            cantons_abbr(field.profile.activity_cantons, abbr=False, long=True)
        )


class CollaboratorUserResource(UserResource):
    """
    Restricted UserResource
    """

    def get_fields(self, **kwargs):
        return [self.fields[f] for f in ALL_COLLABORATOR_FIELDS]



# Generated by Django 2.2.12 on 2020-05-14 13:59

import django.db.models.deletion
from django.conf import settings
from django.contrib.auth import get_user_model
from django.db import migrations, models
from rolepermissions.roles import assign_role


def migrate_organization(apps, schema_editor):
    Organization = apps.get_model("orga", "Organization")
    User = get_user_model()
    EmailAddress = apps.get_model("account", "EmailAddress")
    UserProfile = apps.get_model("user", "UserProfile")
    Address = apps.get_model("common", "Address")

    for orga in Organization.objects.all():
        coordinator_names = orga.coordinator_fullname.split(" ")
        if coordinator_names or orga.coordinator_email:
            user, create = User.objects.get_or_create(
                email=orga.coordinator_email,
                is_active=True,
                defaults={
                    "first_name": coordinator_names[0]
                    if len(coordinator_names) > 0
                    else "",
                    "last_name": " ".join(coordinator_names[1:])
                    if len(coordinator_names) > 1
                    else "",
                },
            )
            if create:
                user.set_unusable_password()
                address_ptr = Address.objects.create()
                UserProfile.objects.create(
                    user_id=user.id,
                    natel=orga.coordinator_natel,
                    phone=orga.coordinator_phone,
                    status=10,  # USERSTATUS_ACTIVE
                    address_ptr_id=address_ptr.id,
                )
                email_address, _ = EmailAddress.objects.get_or_create(
                    email=user.email, defaults={"user_id": user.id}
                )
                email_address.user_id = user.id
                email_address.verified = True
                email_address.primary = True
                email_address.save()

            user.is_active = True
            user.save()
            orga.coordinator_id = user.id
            orga.save()
            assign_role(user, "coordinator")


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("orga", "0006_auto_20170831_1201"),
        ("user", "0059_userprofile_phone"),
    ]

    operations = [
        migrations.AddField(
            model_name="organization",
            name="coordinator",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="managed_organizations",
                to=settings.AUTH_USER_MODEL,
                verbose_name="Coordina·teur·trice",
            ),
        ),
        migrations.RunPython(migrate_organization),
    ]



# defivelo-intranet -- Outil métier pour la gestion du Défi Vélo
# Copyright (C) 2016 Didier Raboud <me+defivelo@odyx.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django import forms
from django.utils.translation import gettext_lazy as _

from apps.user import FORMATION_M1, FORMATION_M2, formation_short

from . import (
    CHOSEN_AS_ACTOR,
    CHOSEN_AS_HELPER,
    CHOSEN_AS_LEADER,
    CHOSEN_AS_LEGACY,
    CHOSEN_AS_NOT,
    CHOSEN_AS_REPLACEMENT,
)
from .models.availability import HelperSessionAvailability


class BSChoiceRadioSelect(forms.RadioSelect):
    template_name = "widgets/BSRadioSelect.html"
    option_template_name = "widgets/BSChoiceRadioSelect_option.html"

    def __init__(self, *args, **kwargs):
        # Trick to pass the 'at which role that user is
        # selected in that quali' information through
        self.user_assignment = kwargs.pop("user_assignment", False)
        return super(BSChoiceRadioSelect, self).__init__(*args, **kwargs)

    def get_context(self, name, value, attrs):
        context = super(BSChoiceRadioSelect, self).get_context(name, value, attrs)
        # Avoid journal messages
        context["widget"]["forbid_absence"] = None
        # User has a status in the session, forbid change
        disable_all = self.user_assignment is not None
        for optgroup in context["widget"]["optgroups"]:
            (group, options, index) = optgroup
            # Avoid journal messages
            options[0]["glyphicon"] = None
            if options[0]["value"] == CHOSEN_AS_LEADER:
                options[0]["text"] = formation_short(FORMATION_M2)
                options[0]["class"] = "success"
            elif options[0]["value"] == CHOSEN_AS_HELPER:
                options[0]["text"] = formation_short(FORMATION_M1)
                options[0]["class"] = "success"
            elif options[0]["value"] == CHOSEN_AS_REPLACEMENT:
                options[0]["text"] = _("S")
                options[0]["class"] = "warning"
            elif options[0]["value"] == CHOSEN_AS_ACTOR:
                options[0]["glyphicon"] = "sunglasses"
                options[0]["class"] = "success"
            elif options[0]["value"] == CHOSEN_AS_LEGACY:
                options[0]["glyphicon"] = "ok-sign"
                options[0]["class"] = "warning"
            elif options[0]["value"] == CHOSEN_AS_NOT:
                options[0]["glyphicon"] = "remove-circle"
                options[0]["class"] = "default"
            options[0]["disabled"] = disable_all
        return context


class BSAvailabilityRadioSelect(forms.RadioSelect):
    template_name = "widgets/BSRadioSelect.html"
    option_template_name = "widgets/BSRadioSelect_option.html"

    def __init__(self, *args, **kwargs):
        self.forbid_absence = kwargs.pop("forbid_absence", False)
        return super(BSAvailabilityRadioSelect, self).__init__(*args, **kwargs)

    def get_context(self, name, value, attrs):
        context = super(BSAvailabilityRadioSelect, self).get_context(name, value, attrs)
        context["widget"]["forbid_absence"] = self.forbid_absence
        return context


class SessionChoiceField(object):
    def __init__(self, *args, **kwargs):
        self.session = kwargs.pop("session", False)
        return super(SessionChoiceField, self).__init__(*args, **kwargs)

    def get_chosen_as(self, user):
        if not self.session:
            return
        try:
            return user.availabilities.get(session=self.session).chosen_as
        except HelperSessionAvailability.DoesNotExist:
            pass

    def if_replacement(self, user):
        return (
            " (%s)" % _("S")
            if self.get_chosen_as(user) == CHOSEN_AS_REPLACEMENT
            else ""
        )


class LeaderChoiceField(SessionChoiceField, forms.ModelChoiceField):
    def label_from_instance(self, obj):
        return "%s%s" % (obj.get_full_name(), self.if_replacement(obj))


class HelpersChoiceField(SessionChoiceField, forms.ModelMultipleChoiceField):
    def label_from_instance(self, obj):
        return "%s%s%s" % (
            obj.get_full_name(),
            (
                " (%s)" % obj.profile.formation
                if obj.profile.formation not in ["", FORMATION_M1]
                else ""
            ),
            self.if_replacement(obj),
        )


class ActorChoiceField(SessionChoiceField, forms.ModelChoiceField):
    def label_from_instance(self, obj):
        return "{name} ({actor_for}){replacement}".format(
            name=obj.get_full_name(),
            actor_for=obj.profile.actor_inline,
            replacement=self.if_replacement(obj),
        )



from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("challenge", "0021_auto_20150917_1730"),
    ]

    operations = [
        migrations.AddField(
            model_name="session",
            name="chosen_staff",
            field=models.ManyToManyField(
                related_name="sessions",
                blank=True,
                verbose_name="Personnel",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.AlterField(
            model_name="qualification",
            name="activity_A",
            field=models.ForeignKey(
                null=True,
                related_name="qualifs_A",
                to="challenge.QualificationActivity",
                blank=True,
                verbose_name="Vélo dans la rue",
                on_delete=models.CASCADE,
            ),
        ),
        migrations.AlterField(
            model_name="qualification",
            name="activity_B",
            field=models.ForeignKey(
                null=True,
                related_name="qualifs_B",
                to="challenge.QualificationActivity",
                blank=True,
                verbose_name="Mécanique",
                on_delete=models.CASCADE,
            ),
        ),
        migrations.AlterField(
            model_name="qualification",
            name="activity_C",
            field=models.ForeignKey(
                null=True,
                related_name="qualifs_C",
                to="challenge.QualificationActivity",
                blank=True,
                verbose_name="Rencontre",
                on_delete=models.CASCADE,
            ),
        ),
        migrations.AlterField(
            model_name="qualification",
            name="actor",
            field=models.ForeignKey(
                null=True,
                related_name="qualifs_actor",
                to=settings.AUTH_USER_MODEL,
                blank=True,
                verbose_name="Intervenant",
                on_delete=models.CASCADE,
            ),
        ),
        migrations.AlterField(
            model_name="qualification",
            name="helpers",
            field=models.ManyToManyField(
                related_name="qualifs_mon1",
                blank=True,
                verbose_name="Moniteurs 1",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.AlterField(
            model_name="qualification",
            name="leader",
            field=models.ForeignKey(
                null=True,
                related_name="qualifs_mon2",
                to=settings.AUTH_USER_MODEL,
                blank=True,
                verbose_name="Moniteur 2",
                on_delete=models.CASCADE,
            ),
        ),
    ]



# defivelo-intranet -- Outil métier pour la gestion du Défi Vélo
# Copyright (C) 2020 Didier Raboud <didier.raboud@liip.ch>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django import forms
from django.utils.translation import gettext_lazy as _

from bootstrap3_datetime.widgets import DateTimePicker
from localflavor.ch.forms import CHStateSelect

from apps.common import DV_STATE_CHOICES

from ..models import AnnualStateSetting


class AnnualStateSettingForm(forms.ModelForm):
    year = forms.IntegerField(
        label=_("Année"),
        widget=DateTimePicker({"placeholder": "YYYY"}, options={"format": "YYYY"}),
        required=True,
    )
    canton = forms.ChoiceField(
        label=_("Canton"),
        widget=CHStateSelect,
        choices=DV_STATE_CHOICES,
        required=True,
    )

    class Meta:
        model = AnnualStateSetting
        fields = ["year", "canton", "cost_per_bike", "cost_per_participant"]

    def __init__(self, *args, **kwargs):
        year = kwargs.pop("year")
        cantons = kwargs.pop("cantons", None)
        super().__init__(*args, **kwargs)
        self.initial["year"] = year
        if cantons:
            # Only permit edition within the allowed cantons
            choices = self.fields["canton"].choices
            choices = ((k, v) for (k, v) in choices if k in cantons)
            self.fields["canton"].choices = choices



# defivelo-intranet -- Outil métier pour la gestion du Défi Vélo
# Copyright (C) 2015 Didier Raboud <me+defivelo@odyx.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.conf.urls import include
from django.urls import re_path
from django.views.decorators.cache import never_cache

from .views import (
    AnnualStateSettingCreateView,
    AnnualStateSettingsListView,
    AnnualStateSettingUpdateView,
    InvoiceCreateView,
    InvoiceDetailView,
    InvoiceListView,
    InvoiceUpdateView,
    InvoiceYearlyListView,
    QualiCreateView,
    QualiDeleteView,
    QualiUpdateView,
    SeasonActorListView,
    SeasonAvailabilityReminderView,
    SeasonAvailabilityUpdateView,
    SeasonAvailabilityView,
    SeasonCreateView,
    SeasonDetailView,
    SeasonErrorsListView,
    SeasonExportView,
    SeasonHelperListView,
    SeasonListRedirect,
    SeasonListView,
    SeasonOrgaListView,
    SeasonPersonalPlanningExportFeed,
    SeasonPersonalPlanningExportView,
    SeasonPlanningExportView,
    SeasonPlanningView,
    SeasonStaffChoiceUpdateView,
    SeasonToOpenView,
    SeasonToRunningView,
    SeasonUpdateView,
    SessionCreateView,
    SessionDeleteView,
    SessionDetailView,
    SessionExportView,
    SessionsListView,
    SessionStaffChoiceView,
    SessionUpdateView,
)
from .views.invoice import InvoiceListExport
from .views.registration import register, register_confirm, register_validate
from .views.season import SeasonGeneralPlanningView
from .views.session import SessionCloneView

urlpatterns = [
    # Settings
    re_path(
        "^settings/y(?P<year>[0-9]{4})/",
        include(
            [
                re_path(
                    r"^(?P<pk>[0-9]+)/$",
                    never_cache(AnnualStateSettingUpdateView.as_view()),
                    name="annualstatesetting-update",
                ),
                re_path(
                    "^new/$",
                    never_cache(AnnualStateSettingCreateView.as_view()),
                    name="annualstatesetting-create",
                ),
                re_path(
                    "^$",
                    never_cache(AnnualStateSettingsListView.as_view()),
                    name="annualstatesettings-list",
                ),
            ]
        ),
    ),
    # Seasons
    re_path(
        r"^$",
        SeasonListRedirect.as_view(),
        name="season-list",
    ),
    re_path(
        r"^y(?P<year>[0-9]{4})/",
        include(
            [
                re_path(
                    r"^invoices/$",
                    never_cache(InvoiceYearlyListView.as_view()),
                    name="invoices-yearly-list",
                ),
                re_path(
                    r"^invoices/(?P<void>)?(?P<format>[a-z]+)export/$",
                    never_cache(InvoiceListExport.as_view()),
                    name="invoices-yearly-list-export",
                ),
                re_path(
                    r"(?P<dv_season>[0-4]{1})/$",
                    never_cache(SeasonListView.as_view()),
                    name="season-list",
                ),
                re_path(
                    r"(?P<dv_season>[0-4]{1})/general/(?P<helperpk>[0-9]+)/$",
                    never_cache(SeasonGeneralPlanningView.as_view()),
                    name="season-general-planning",
                ),
                re_path(
                    r"(?P<dv_season>[0-4]{1})/general/(?P<helperpk>[0-9]+)/(?P<format>[a-z]+)exportplanning$",
                    never_cache(SeasonPersonalPlanningExportView.as_view()),
                    name="season-personal-planning-export",
                ),
                re_path(
                    r"(?P<dv_season>[0-4]{1})/general/(?P<helperpk>[0-9]+)/feed\.ics$",
                    SeasonPersonalPlanningExportFeed(),
                    name="season-personal-calendar",
                ),
            ]
        ),
    ),
    re_path(r"^new/$", SeasonCreateView.as_view(), name="season-create"),
    re_path(
        r"^(?P<pk>[0-9]+)/",
        include(
            [
                re_path(r"^update/$", SeasonUpdateView.as_view(), name="season-update"),
                re_path(
                    r"^running-planning/$",
                    SeasonToRunningView.as_view(),
                    name="season-set-running",
                ),
                re_path(
                    r"^open-planning/$",
                    SeasonToOpenView.as_view(),
                    name="season-set-open",
                ),
                re_path(
                    r"^$",
                    never_cache(SeasonDetailView.as_view()),
                    name="season-detail",
                ),
                re_path(
                    r"^(?P<format>[a-z]+)export$",
                    never_cache(SeasonExportView.as_view()),
                    name="season-export",
                ),
                re_path(
                    r"^(?P<format>[a-z]+)exportplanning$",
                    never_cache(SeasonPlanningExportView.as_view()),
                    name="season-planning-export",
                ),
                re_path(
                    r"^moniteurs/$",
                    never_cache(SeasonHelperListView.as_view()),
                    name="season-helperlist",
                ),
                re_path(
                    r"^intervenants/$",
                    never_cache(SeasonActorListView.as_view()),
                    name="season-actorlist",
                ),
                re_path(
                    r"^erreurs/$",
                    never_cache(SeasonErrorsListView.as_view()),
                    name="season-errorslist",
                ),
                re_path(
                    r"^availability/$",
                    never_cache(SeasonAvailabilityView.as_view()),
                    name="season-availabilities",
                ),
                re_path(
                    r"^availability/reminder/$",
                    never_cache(SeasonAvailabilityReminderView.as_view()),
                    name="season-availability-reminder",
                ),
                re_path(
                    r"^planning/(?P<helperpk>[0-9]+)/",
                    include(
                        [
                            re_path(
                                "^$",
                                never_cache(SeasonPlanningView.as_view()),
                                name="season-planning",
                            ),
                            re_path(
                                "^(?P<format>[a-z]+)exportplanning$",
                                never_cache(SeasonPersonalPlanningExportView.as_view()),
                                name="season-personal-planning-export",
                            ),
                            re_path(
                                r"^feed.ics$",
                                SeasonPersonalPlanningExportFeed(),
                                name="season-personal-calendar",
                            ),
                        ]
                    ),
                ),
                re_path(
                    r"^availability/staff/$",
                    never_cache(SeasonStaffChoiceUpdateView.as_view()),
                    name="season-staff-update",
                ),
                re_path(
                    r"^availability/(?P<helperpk>[0-9]+)/$",
                    SeasonAvailabilityUpdateView.as_view(),
                    name="season-availabilities-update",
                ),
            ]
        ),
    ),
    # Invoices
    re_path(
        r"^(?P<seasonpk>[0-9]+)/",
        include(
            [
                re_path(
                    "invoices/",
                    never_cache(SeasonOrgaListView.as_view()),
                    name="invoice-orga-list",
                ),
                re_path(
                    r"i(?P<orgapk>[0-9]+)/",
                    include(
                        [
                            re_path(
                                r"^new/$",
                                never_cache(InvoiceCreateView.as_view()),
                                name="invoice-create",
                            ),
                            re_path(
                                r"^list/$",
                                never_cache(InvoiceListView.as_view()),
                                name="invoice-list",
                            ),
                            re_path(
                                r"^(?P<invoiceref>.+)/edit/$",
                                never_cache(InvoiceUpdateView.as_view()),
                                name="invoice-update",
                            ),
                            re_path(
                                r"^(?P<invoiceref>.+)/$",
                                never_cache(InvoiceDetailView.as_view()),
                                name="invoice-detail",
                            ),
                        ]
                    ),
                ),
            ]
        ),
    ),
    # Sessions
    re_path(
        r"^(?P<seasonpk>[0-9]+)/(?P<year>[0-9]{4})/w(?P<week>[0-9]+)/$",
        never_cache(SessionsListView.as_view()),
        name="session-list",
    ),
    re_path(
        r"^(?P<seasonpk>[0-9]+)/snew/$",
        never_cache(SessionCreateView.as_view()),
        name="session-create",
    ),
    re_path(
        r"^(?P<seasonpk>[0-9]+)/s(?P<pk>[0-9]+)/$",
        never_cache(SessionDetailView.as_view()),
        name="session-detail",
    ),
    re_path(
        r"^(?P<seasonpk>[0-9]+)/s(?P<pk>[0-9]+)/(?P<format>[a-z]+)export$",
        never_cache(SessionExportView.as_view()),
        name="session-export",
    ),
    re_path(
        r"^(?P<seasonpk>[0-9]+)/s(?P<pk>[0-9]+)/availability/$",
        never_cache(SessionStaffChoiceView.as_view()),
        name="session-staff-choices",
    ),
    re_path(
        r"^(?P<seasonpk>[0-9]+)/s(?P<pk>[0-9]+)/update/$",
        SessionUpdateView.as_view(),
        name="session-update",
    ),
    re_path(
        r"^(?P<seasonpk>[0-9]+)/s(?P<pk>[0-9]+)/clone/$",
        SessionCloneView.as_view(),
        name="session-clone",
    ),
    re_path(
        r"^(?P<seasonpk>[0-9]+)/s(?P<pk>[0-9]+)/delete/$",
        SessionDeleteView.as_view(),
        name="session-delete",
    ),
    # Qualis
    re_path(
        r"^(?P<seasonpk>[0-9]+)/s(?P<sessionpk>[0-9]+)/quali$",
        never_cache(QualiCreateView.as_view()),
        name="quali-create",
    ),
    re_path(
        r"^(?P<seasonpk>[0-9]+)/s(?P<sessionpk>[0-9]+)/q(?P<pk>[0-9]+)/$",
        never_cache(QualiUpdateView.as_view()),
        name="quali-update",
    ),
    re_path(
        r"^(?P<seasonpk>[0-9]+)/s(?P<sessionpk>[0-9]+)/q(?P<pk>[0-9]+)/delete$",
        QualiDeleteView.as_view(),
        name="quali-delete",
    ),
    # Registrations
    re_path(r"^registration/$", register, name="registration-create"),
    re_path(r"^registration/confirm/$", register_confirm, name="registration-confirm"),
    re_path(
        r"^registrations/validate/$", register_validate, name="registration-validate"
    ),
]



from django.conf import settings
from django.db import models
from django.utils.translation import gettext_lazy as _

from .. import BONUS_LEADER, HOURLY_RATE_HELPER, RATE_ACTOR


class Timesheet(models.Model):
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        related_name="timesheets",
        on_delete=models.CASCADE,
    )
    date = models.DateField(_("Date"), blank=True, null=True)

    time_helper = models.FloatField(_("Heures moni·teur·trice"), default=0)
    actor_count = models.IntegerField(_("Intervention(s)"), default=0)
    leader_count = models.IntegerField(
        _("Participation(s) comme moniteur·trice 2"), default=0
    )

    overtime = models.FloatField(_("Heures supplémentaires"), default=0)
    traveltime = models.FloatField(_("Heures de trajet"), default=0)
    comments = models.TextField(_("Remarques"), blank=True)

    validated_at = models.DateTimeField(null=True, blank=True)
    validated_by = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        related_name="+",
        on_delete=models.SET_NULL,
        null=True,
    )
    ignore = models.BooleanField(_("Ignorer"), default=False)

    class Meta:
        unique_together = (
            (
                "user",
                "date",
            ),
        )

    def get_total_amount_helper(self):
        if self.ignore:
            return 0
        return (
            (self.time_helper or 0) + self.overtime + self.traveltime
        ) * HOURLY_RATE_HELPER

    def get_total_amount_actor(self):
        if self.ignore:
            return 0
        return (self.actor_count or 0) * RATE_ACTOR

    def get_total_amount_leader(self):
        if self.ignore:
            return 0
        return (self.leader_count or 0) * BONUS_LEADER

    def get_total_amount(self):
        if self.ignore:
            return 0
        return (
            self.get_total_amount_actor()
            + self.get_total_amount_helper()
            + self.get_total_amount_leader()
        )



import datetime

from django.urls import reverse

from apps.challenge.tests.factories import (
    QualificationFactory,
    SeasonFactory,
    SessionFactory,
)
from apps.orga.tests.factories import OrganizationFactory
from apps.salary.models import Timesheet
from apps.user.tests.factories import UserFactory
from defivelo.roles import user_cantons
from defivelo.tests.utils import (
    AuthClient,
    CollaboratorAuthClient,
    StateManagerAuthClient,
)


def test_authed_user_cannot_see_timesheet(db):
    client = AuthClient()

    SeasonFactory(
        cantons=["VD"],
        year=2019,
        month_start=1,
        n_months=5,
    )
    QualificationFactory(
        actor=client.user,
        session=SessionFactory(
            day=datetime.date(2019, 4, 12),
            orga=OrganizationFactory(address_canton="VD"),
        ),
    )

    response = client.get(
        reverse(
            "salary:user-timesheets",
            kwargs={"year": 2019, "month": 4, "pk": client.user.pk},
        )
    )

    assert response.status_code == 403  # Forbidden


def test_collaborator_can_see_his_timesheet(db):
    client = CollaboratorAuthClient()

    SeasonFactory(
        cantons=["VD"],
        year=2019,
        month_start=1,
        n_months=5,
    )
    QualificationFactory(
        actor=client.user,
        session=SessionFactory(
            day=datetime.date(2019, 4, 12),
            orga=OrganizationFactory(address_canton="VD"),
        ),
    )

    response = client.get(
        reverse(
            "salary:user-timesheets",
            kwargs={"year": 2019, "month": 4, "pk": client.user.pk},
        )
    )

    assert client.user.get_full_name() in response.content.decode()
    assert 1 == len(response.context["form"].forms)


def test_collaborator_cannot_timesheet_overtime_without_comments(db):
    client = CollaboratorAuthClient()

    SeasonFactory(
        cantons=["VD"],
        year=2019,
        month_start=1,
        n_months=5,
    )
    QualificationFactory(
        actor=client.user,
        session=SessionFactory(
            day=datetime.date(2019, 4, 12),
            orga=OrganizationFactory(address_canton="VD"),
        ),
    )

    datas = {
        "form-TOTAL_FORMS": "1",
        "form-INITIAL_FORMS": "1",
        "form-MIN_NUM_FORMS": "0",
        "form-MAX_NUM_FORMS": "0",
        "form-0-date": "2019-04-12",
        "form-0-time_helper": "4.5",
        "form-0-actor_count": "0",
        "form-0-leader_count": "0",
        "form-0-overtime": "0.25",
        "form-0-traveltime": "0.25",
        "form-0-comments": "",
    }

    client.post(
        reverse(
            "salary:user-timesheets",
            kwargs={"year": 2019, "month": 4, "pk": client.user.pk},
        ),
        datas,
    )
    assert Timesheet.objects.count() == 0


def test_collaborator_can_timesheet(db):
    client = CollaboratorAuthClient()

    SeasonFactory(
        cantons=["VD"],
        year=2019,
        month_start=1,
        n_months=5,
    )
    QualificationFactory(
        actor=client.user,
        session=SessionFactory(
            day=datetime.date(2019, 4, 12),
            orga=OrganizationFactory(address_canton="VD"),
        ),
    )

    datas = {
        "form-TOTAL_FORMS": "1",
        "form-INITIAL_FORMS": "1",
        "form-MIN_NUM_FORMS": "0",
        "form-MAX_NUM_FORMS": "0",
        "form-0-date": "2019-04-12",
        "form-0-time_helper": "4.5",
        "form-0-actor_count": "0",
        "form-0-leader_count": "0",
        "form-0-overtime": "0.25",
        "form-0-traveltime": "0.25",
        "form-0-comments": "I took longer",
    }

    client.post(
        reverse(
            "salary:user-timesheets",
            kwargs={"year": 2019, "month": 4, "pk": client.user.pk},
        ),
        datas,
    )
    assert Timesheet.objects.count() == 1


def test_collaborator_can_update_timesheet(db):
    client = CollaboratorAuthClient()

    SeasonFactory(
        cantons=["VD"],
        year=2019,
        month_start=1,
        n_months=5,
    )
    QualificationFactory(
        actor=client.user,
        session=SessionFactory(
            day=datetime.date(2019, 4, 12),
            orga=OrganizationFactory(address_canton="VD"),
        ),
    )

    datas = {
        "form-TOTAL_FORMS": "1",
        "form-INITIAL_FORMS": "1",
        "form-MIN_NUM_FORMS": "0",
        "form-MAX_NUM_FORMS": "0",
        "form-0-date": "2019-04-12",
        "form-0-time_helper": "4.5",
        "form-0-actor_count": "0",
        "form-0-leader_count": "0",
        "form-0-overtime": "0.25",
        "form-0-traveltime": "0.25",
        "form-0-comments": "Initial comment",
    }

    client.post(
        reverse(
            "salary:user-timesheets",
            kwargs={"year": 2019, "month": 4, "pk": client.user.pk},
        ),
        datas,
    )
    datas["form-0-overtime"] = 1
    datas["form-0-comments"] = "New comment"
    client.post(
        reverse(
            "salary:user-timesheets",
            kwargs={"year": 2019, "month": 4, "pk": client.user.pk},
        ),
        datas,
    )
    assert (
        Timesheet.objects.count() == 1
        and Timesheet.objects.first().overtime == 1
        and Timesheet.objects.first().comments == "New comment"
    )


def test_collaborator_cannot_validate_timesheet(db):
    client = CollaboratorAuthClient()

    SeasonFactory(
        cantons=["VD"],
        year=2019,
        month_start=1,
        n_months=5,
    )
    QualificationFactory(
        actor=client.user,
        session=SessionFactory(
            day=datetime.date(2019, 4, 12),
            orga=OrganizationFactory(address_canton="VD"),
        ),
    )

    datas = {
        "form-TOTAL_FORMS": "1",
        "form-INITIAL_FORMS": "1",
        "form-MIN_NUM_FORMS": "0",
        "form-MAX_NUM_FORMS": "0",
        "form-0-date": "2019-04-12",
        "form-0-time_helper": "4.5",
        "form-0-actor_count": "0",
        "form-0-leader_count": "0",
        "form-0-overtime": "0.25",
        "form-0-traveltime": "0.25",
        "form-0-comments": "Comment",
        "form-0-validated": True,
    }

    client.post(
        reverse(
            "salary:user-timesheets",
            kwargs={"year": 2019, "month": 4, "pk": client.user.pk},
        ),
        datas,
    )
    assert (
        Timesheet.objects.count() == 1
        and Timesheet.objects.first().validated_at is None
    )


def test_collaborator_cannot_set_timesheet_to_ignore(db):
    client = CollaboratorAuthClient()

    SeasonFactory(cantons=["VD"], year=2019, month_start=1, n_months=5)
    QualificationFactory(
        actor=client.user,
        session=SessionFactory(
            day=datetime.date(2019, 4, 12),
            orga=OrganizationFactory(address_canton="VD"),
        ),
    )

    datas = {
        "form-TOTAL_FORMS": "1",
        "form-INITIAL_FORMS": "1",
        "form-MIN_NUM_FORMS": "0",
        "form-MAX_NUM_FORMS": "0",
        "form-0-date": "2019-04-12",
        "form-0-time_helper": "4.5",
        "form-0-actor_count": "0",
        "form-0-leader_count": "0",
        "form-0-overtime": "0.25",
        "form-0-traveltime": "0.25",
        "form-0-comments": "Comment",
        "form-0-ignore": True,
    }

    client.post(
        reverse(
            "salary:user-timesheets",
            kwargs={"year": 2019, "month": 4, "pk": client.user.pk},
        ),
        datas,
    )
    assert Timesheet.objects.count() == 1 and not Timesheet.objects.first().ignore


def test_state_manager_can_validate_timesheet(db):
    client = StateManagerAuthClient()
    managed_cantons = user_cantons(client.user)
    actor = UserFactory(
        first_name="Maurice",
        last_name="Moss",
        profile__affiliation_canton=managed_cantons[0],
    )
    SeasonFactory(
        cantons=[managed_cantons[0]],
        year=2019,
        month_start=1,
        n_months=5,
    )
    QualificationFactory(
        actor=actor,
        session=SessionFactory(
            day=datetime.date(2019, 4, 12),
            orga=OrganizationFactory(address_canton=managed_cantons[0]),
        ),
    )

    datas = {
        "form-TOTAL_FORMS": "1",
        "form-INITIAL_FORMS": "1",
        "form-MIN_NUM_FORMS": "0",
        "form-MAX_NUM_FORMS": "0",
        "form-0-date": "2019-04-12",
        "form-0-time_helper": "4.5",
        "form-0-actor_count": "0",
        "form-0-leader_count": "0",
        "form-0-overtime": "0.25",
        "form-0-traveltime": "0.25",
        "form-0-comments": "Comment",
        "form-0-validated": True,
    }

    client.post(
        reverse(
            "salary:user-timesheets",
            kwargs={"year": 2019, "month": 4, "pk": actor.pk},
        ),
        datas,
    )
    assert Timesheet.objects.count() == 1 and Timesheet.objects.first().validated_at


def test_state_manager_can_set_timesheet_to_ignore(db):
    client = StateManagerAuthClient()
    managed_cantons = user_cantons(client.user)
    actor = UserFactory(
        first_name="Maurice",
        last_name="Moss",
        profile__affiliation_canton=managed_cantons[0],
    )
    SeasonFactory(cantons=[managed_cantons[0]], year=2019, month_start=1, n_months=5)
    QualificationFactory(
        actor=actor,
        session=SessionFactory(
            day=datetime.date(2019, 4, 12),
            orga=OrganizationFactory(address_canton=managed_cantons[0]),
        ),
    )

    datas = {
        "form-TOTAL_FORMS": "1",
        "form-INITIAL_FORMS": "1",
        "form-MIN_NUM_FORMS": "0",
        "form-MAX_NUM_FORMS": "0",
        "form-0-date": "2019-04-12",
        "form-0-time_helper": "4.5",
        "form-0-actor_count": "0",
        "form-0-leader_count": "0",
        "form-0-overtime": "0.25",
        "form-0-traveltime": "0.25",
        "form-0-comments": "Comment",
        "form-0-ignore": True,
    }

    client.post(
        reverse(
            "salary:user-timesheets",
            kwargs={"year": 2019, "month": 4, "pk": actor.pk},
        ),
        datas,
    )
    assert Timesheet.objects.count() == 1


def test_state_manager_forms_disabled_for_unmanaged_canton(db):
    client = StateManagerAuthClient()
    managed_canton = user_cantons(client.user)[0]  # VD
    unmanaged_canton = "GE" if managed_canton != "GE" else "VD"

    SeasonFactory(cantons=[managed_canton], year=2019, month_start=1, n_months=5)
    SeasonFactory(cantons=[unmanaged_canton], year=2019, month_start=1, n_months=5)

    actor = UserFactory(profile__affiliation_canton=managed_canton)

    managed_session = SessionFactory(
        day=datetime.date(2019, 4, 10),
        orga=OrganizationFactory(address_canton=managed_canton),
    )
    unmanaged_session = SessionFactory(
        day=datetime.date(2019, 4, 11),
        orga=OrganizationFactory(address_canton=unmanaged_canton),
    )
    QualificationFactory(actor=actor, session=managed_session)
    QualificationFactory(actor=actor, session=unmanaged_session)

    response = client.get(
        reverse(
            "salary:user-timesheets",
            kwargs={"year": 2019, "month": 4, "pk": actor.pk},
        )
    )
    assert response.status_code == 200

    formset = response.context["form"]
    assert getattr(formset.forms[0], "disabled", False) is False
    assert getattr(formset.forms[1], "disabled", False) is True
    assert getattr(formset, "has_disabled_forms", False) is True


def test_state_manager_can_validate_own_timesheet_in_unmanaged_canton(db):
    client = StateManagerAuthClient()
    managed_canton = user_cantons(client.user)[0]
    unmanaged_canton = "GE" if managed_canton != "GE" else "VD"

    SeasonFactory(cantons=[managed_canton], year=2019, month_start=1, n_months=5)
    SeasonFactory(cantons=[unmanaged_canton], year=2019, month_start=1, n_months=5)

    managed_session = SessionFactory(
        day=datetime.date(2019, 4, 10),
        orga=OrganizationFactory(address_canton=managed_canton),
    )
    unmanaged_session = SessionFactory(
        day=datetime.date(2019, 4, 11),
        orga=OrganizationFactory(address_canton=unmanaged_canton),
    )
    QualificationFactory(actor=client.user, session=managed_session)
    QualificationFactory(actor=client.user, session=unmanaged_session)

    response = client.get(
        reverse(
            "salary:user-timesheets",
            kwargs={"year": 2019, "month": 4, "pk": client.user.pk},
        )
    )
    assert response.status_code == 200

    formset = response.context["form"]
    assert getattr(formset.forms[0], "disabled", False) is False
    assert getattr(formset.forms[1], "disabled", False) is False
    assert getattr(formset, "has_disabled_forms", False) is False


def test_collaborator_cannot_timesheet_if_he_has_not_work(db):
    client = CollaboratorAuthClient()
    datas = {
        "form-TOTAL_FORMS": "1",
        "form-INITIAL_FORMS": "1",
        "form-MIN_NUM_FORMS": "0",
        "form-MAX_NUM_FORMS": "0",
        "form-0-date": "2019-04-12",
        "form-0-time_helper": "4.5",
        "form-0-actor_count": "0",
        "form-0-leader_count": "0",
        "form-0-overtime": "0.25",
        "form-0-traveltime": "0.25",
        "form-0-comments": "Comment",
    }

    client.post(
        reverse(
            "salary:user-timesheets",
            kwargs={"year": 2019, "month": 4, "pk": client.user.pk},
        ),
        datas,
    )
    assert Timesheet.objects.count() == 0



# defivelo-intranet -- Outil métier pour la gestion du Défi Vélo
# Copyright (C) 2015, 2016, 2020 Didier Raboud <didier.raboud@liip.ch>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from datetime import date

from django.db.models import F
from django.views.generic.base import TemplateView

from stronghold.views import StrongholdPublicMixin

from apps.article.models import Article
from apps.common import DV_SEASON_AUTUMN, DV_SEASON_LAST_SPRING_MONTH, DV_SEASON_SPRING
from defivelo.roles import has_permission


class MenuView(object):
    def current_season(self):
        return (
            DV_SEASON_SPRING
            if date.today().month <= DV_SEASON_LAST_SPRING_MONTH
            else DV_SEASON_AUTUMN
        )

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        today = date.today()

        # Uniquement les saisons==mois qui couvrent le jour courant
        context["current_seasons"] = self.request.user.profile.get_seasons().filter(
            year=today.year,
            month_start__lte=today.month,
            n_months__gt=today.month - F("month_start"),
        )
        return context


class HomeView(MenuView, TemplateView):
    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        if not has_permission(self.request.user, "coordinator_home"):
            articles_qs = Article.objects
            if not has_permission(self.request.user, "home_article_cud"):
                articles_qs = articles_qs.filter(published=True)
            context["articles"] = articles_qs.order_by("-modified")[:5]
        # Add our menu_category context
        context["menu_category"] = "home"
        return context

    def get_template_names(self):
        if has_permission(self.request.user, "coordinator_home"):
            self.template_name = "coordinator_home.html"
        else:
            self.template_name = "home.html"

        return super().get_template_names()


class LicenseView(StrongholdPublicMixin, TemplateView):
    template_name = "license.html"



from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("challenge", "0083_historicalqualification_n_helpers_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="season",
            name="availability_reminder_sent_at",
            field=models.DateTimeField(
                verbose_name="Date d’envoi du rappel de disponibilités",
                null=True,
                blank=True,
            ),
        ),
    ]





from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("challenge", "0017_auto_20150914_1658"),
    ]

    operations = [
        migrations.CreateModel(
            name="Season",
            fields=[
                (
                    "id",
                    models.AutoField(
                        serialize=False,
                        primary_key=True,
                        auto_created=True,
                        verbose_name="ID",
                    ),
                ),
                ("created_on", models.DateTimeField(auto_now_add=True)),
                ("begin", models.DateField(verbose_name="Début")),
                ("end", models.DateField(verbose_name="Fin")),
                (
                    "cantons",
                    models.CharField(
                        choices=[
                            ("AG", "Aargau"),
                            ("AI", "Appenzell Innerrhoden"),
                            ("AR", "Appenzell Ausserrhoden"),
                            ("BS", "Basel-Stadt"),
                            ("BL", "Basel-Land"),
                            ("BE", "Berne"),
                            ("FR", "Fribourg"),
                            ("GE", "Geneva"),
                            ("GL", "Glarus"),
                            ("GR", "Graubuenden"),
                            ("JU", "Jura"),
                            ("LU", "Lucerne"),
                            ("NE", "Neuchatel"),
                            ("NW", "Nidwalden"),
                            ("OW", "Obwalden"),
                            ("SH", "Schaffhausen"),
                            ("SZ", "Schwyz"),
                            ("SO", "Solothurn"),
                            ("SG", "St. Gallen"),
                            ("TG", "Thurgau"),
                            ("TI", "Ticino"),
                            ("UR", "Uri"),
                            ("VS", "Valais"),
                            ("VD", "Vaud"),
                            ("ZG", "Zug"),
                            ("ZH", "Zurich"),
                        ],
                        max_length=77,
                        verbose_name="Cantons",
                    ),
                ),
                (
                    "leader",
                    models.ForeignKey(
                        null=True,
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="Chargé de projet",
                        blank=True,
                        on_delete=models.CASCADE,
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "Saison",
                "ordering": ["begin", "end"],
                "verbose_name": "Saison",
            },
        ),
    ]



# defivelo-intranet -- Outil métier pour la gestion du Défi Vélo
# Copyright (C) 2016 Didier Raboud <me+defivelo@odyx.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import operator
from functools import reduce
from urllib.parse import parse_qs, urlencode, urlparse, urlunparse

from django.contrib.auth import get_user_model
from django.contrib.messages.views import SuccessMessageMixin
from django.db.models import Q
from django.urls import reverse_lazy
from django.utils.http import url_has_allowed_host_and_scheme
from django.utils.translation import gettext_lazy as _
from django.views.generic import DetailView
from django.views.generic.edit import CreateView, UpdateView

from django_filters import (
    CharFilter,
    FilterSet,
    ModelMultipleChoiceFilter,
    MultipleChoiceFilter,
)
from django_filters.views import FilterView
from rolepermissions.checkers import has_role
from rolepermissions.mixins import HasPermissionsMixin

from apps.challenge.models import QualificationActivity
from apps.common import DV_LANGUAGES_WITH_DEFAULT, DV_STATE_CHOICES_WITH_DEFAULT
from apps.common.views import ExportMixin, PaginatorMixin
from defivelo.roles import DV_AVAILABLE_ROLES, has_permission, user_cantons

from .. import FORMATION_KEYS
from ..export import CollaboratorUserResource, UserResource
from ..forms import SwissDateFilter
from ..models import (
    FORMATION_CHOICES,
    USERSTATUS_ACTIVE,
    USERSTATUS_ARCHIVE,
    USERSTATUS_CHOICES,
    USERSTATUS_DELETED,
    USERSTATUS_RESERVE,
)
from .mixins import ProfileMixin, UserSelfAccessMixin


class ReturnUrlMixin:
    def get_return_url(self):
        return_url = self.request.GET.get("returnUrl") or self.request.POST.get(
            "returnUrl"
        )
        if not return_url or not url_has_allowed_host_and_scheme(
            url=return_url,
            allowed_hosts={self.request.get_host()},
            require_https=self.request.is_secure(),
        ):
            return None

        return return_url

    def get_return_label(self, default=None):
        if not self.get_return_url():
            return None

        result = self.request.GET.get("returnLabel") or self.request.POST.get(
            "returnLabel"
        )
        return result if result else default

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["return_url"] = self.get_return_url()
        context["return_label"] = self.get_return_label()
        return context

    def get_success_url(self):
        return_url = self.get_return_url()
        if return_url:
            return return_url
        return super().get_success_url()

    @staticmethod
    def _add_return_url(url, return_url):
        """
        Add the returnUrl parameter to the given URL
        """
        parsed = urlparse(str(url))

        # Parse existing query parameters
        query_params = parse_qs(parsed.query)

        # Add the returnUrl parameter
        query_params["returnUrl"] = [return_url]

        # Reconstruct the URL with the new parameter
        new_query = urlencode(query_params, doseq=True)
        return urlunparse(
            (
                parsed.scheme,
                parsed.netloc,
                parsed.path,
                parsed.params,
                new_query,
                parsed.fragment,
            )
        )


class UserDetail(UserSelfAccessMixin, ProfileMixin, ReturnUrlMixin, DetailView):
    required_permission = "user_detail_other"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        try:
            managed_cantons = user_cantons(self.request.user)
        except LookupError:
            managed_cantons = []
        user_cantons_intersection = [
            orga.address_canton
            for orga in self.get_object().managed_organizations.all()
            if orga.address_canton in managed_cantons
        ]
        context["userprofilecanton"] = (
            user_cantons_intersection[0] if user_cantons_intersection else None
        )
        context["profile_is_coordinator"] = has_role(self.object, "coordinator")
        context["requester_is_state_manager"] = has_role(
            self.request.user, "state_manager"
        )
        context["user_has_a_role"] = self.get_object().groups.exists()
        return context


class UserUpdate(
    UserSelfAccessMixin, ProfileMixin, SuccessMessageMixin, ReturnUrlMixin, UpdateView
):
    success_message = _("Profil mis à jour")

    def get_initial(self):
        """
        Pre-fill the form with the non-model fields
        """
        user = self.get_object()
        if hasattr(user, "profile"):
            struct = {}
            for field in self.profile_fields:
                struct[field] = getattr(user.profile, field)
            struct["actor_for"] = user.profile.actor_for.all()
            return struct

    def get_success_url(self):
        return_url = self.get_return_url()
        success_url = super().get_success_url()
        if return_url:
            # Add the returnUrl parameter to the success_url
            success_url = self._add_return_url(success_url, return_url)

            # Add the returnLabel parameter if any
            return_label = self.get_return_label()
            if return_label:
                success_url = (
                    success_url
                    + ("&" if "?" in success_url else "?")
                    + urlencode({"returnLabel": return_label})
                )

        return success_url

    def dispatch(self, request, *args, **kwargs):
        return super(UserUpdate, self).dispatch(request, *args, edit=True, **kwargs)


class UserCreate(HasPermissionsMixin, ProfileMixin, SuccessMessageMixin, CreateView):
    required_permission = "user_create"
    success_message = _("Utilisateur·rice créé·e")

    def get_form_kwargs(self):
        kwargs = super(UserCreate, self).get_form_kwargs()
        kwargs["allow_email"] = True
        return kwargs

    def get_success_url(self):
        try:
            return reverse_lazy("user-detail", kwargs={"pk": self.object.pk})
        except Exception:
            return reverse_lazy("user-list")


class UserProfileFilterSet(FilterSet):
    def __init__(self, data=None, *args, request=None, **kwargs):
        any_filter_is_set = bool(set(self.base_filters) & set(data or {}))
        if not any_filter_is_set:
            data = {}
            for name, f in self.base_filters.items():
                initial = f.extra.get("initial")
                # filter param is either missing or empty, use initial as default
                if not data.get(name) and initial:
                    data[name] = initial
        super().__init__(data=data, *args, request=request, **kwargs)

        if not has_permission(request.user, "user_export_all_fields"):
            # That user only has a limited amount of choices available, amend them
            # Remove the inaccessible statuses
            status_choices = self.filters["profile__status"].extra["choices"]
            self.filters["profile__status"].extra["choices"] = (
                t
                for t in status_choices
                if t[0]
                not in [
                    USERSTATUS_ARCHIVE,
                    USERSTATUS_DELETED,
                ]
            )
            # Remove filters that should be displayed to admins only
            del self.filters["roles"]
            del self.filters["profile__updated_at"]

    def filter_multi_nonempty(queryset, name, values):
        if values:
            allor_filter = [Q(**{name: v}) for v in values if v]
            if allor_filter:
                return queryset.filter(reduce(operator.or_, allor_filter))
        return queryset

    def filter_cantons(queryset, name, values):
        if values and any(values):
            # S'il y au moins un canton en commun
            allcantons_filter = [Q(profile__activity_cantons__overlap=values)] + [
                Q(profile__affiliation_canton=v) for v in values if v
            ]
            return queryset.filter(reduce(operator.or_, allcantons_filter))
        return queryset

    def filter_languages(queryset, name, values):
        if values and any(values):
            # S'il y au moins une langue en commun
            alllangs_filter = [Q(profile__languages_challenges__overlap=values)] + [
                Q(profile__language=v) for v in values if v
            ]
            return queryset.filter(reduce(operator.or_, alllangs_filter))
        return queryset

    def filter_wide(queryset, name, value):
        if value:
            allfields_filter = [
                Q(last_name__unaccent__icontains=value),
                Q(first_name__unaccent__icontains=value),
                Q(email__icontains=value),
                Q(profile__natel__icontains=value),
            ]
            return queryset.filter(reduce(operator.or_, allfields_filter))
        return queryset

    def filter_roles(queryset, name, value):
        try:
            # Use the '0' special value (see definition of the role filter) to specially filter the absence of any role
            if int(value[0]) == 0:
                return queryset.filter(groups__isnull=True)
        except (KeyError, ValueError):
            pass
        return queryset.filter(reduce(operator.or_, [Q(groups__name=r) for r in value]))

    def filter_updated_at(queryset, name, value):
        return queryset.filter(profile__updated_at__gte=value)

    profile__language = MultipleChoiceFilter(
        label=_("Langue"),
        choices=DV_LANGUAGES_WITH_DEFAULT,
        method=filter_multi_nonempty,
    )

    profile__languages_challenges = MultipleChoiceFilter(
        label=_("Langues d’animation"),
        choices=DV_LANGUAGES_WITH_DEFAULT,
        method=filter_languages,
    )

    profile__affiliation_canton = MultipleChoiceFilter(
        label=_("Canton d’affiliation"),
        choices=DV_STATE_CHOICES_WITH_DEFAULT,
        method=filter_multi_nonempty,
    )

    profile__activity_cantons = MultipleChoiceFilter(
        label=_("Cantons"), choices=DV_STATE_CHOICES_WITH_DEFAULT, method=filter_cantons
    )
    profile__status = MultipleChoiceFilter(
        label=_("Statut"),
        choices=USERSTATUS_CHOICES,
        initial=[
            USERSTATUS_ACTIVE,
            USERSTATUS_RESERVE,
        ],
    )
    profile__formation = MultipleChoiceFilter(
        label=_("Formation"), choices=FORMATION_CHOICES, method=filter_multi_nonempty
    )
    profile__actor_for = ModelMultipleChoiceFilter(
        label=_("Intervenant·e"),
        queryset=(
            QualificationActivity.objects.filter(category="C").prefetch_related(
                "translations"
            )
        ),
    )

    profile__updated_at = SwissDateFilter(
        label=_("Dernière modification après le"),
        method=filter_updated_at,
    )

    roles = MultipleChoiceFilter(
        label=_("Rôle"),
        choices=tuple((t[0] if t[0] else 0, t[1]) for t in DV_AVAILABLE_ROLES),
        method=filter_roles,
    )
    q = CharFilter(label=_("Recherche"), method=filter_wide)

    class Meta:
        model = get_user_model()
        fields = [
            "profile__status",
            "profile__formation",
            "profile__actor_for",
            "profile__activity_cantons",
            "profile__updated_at",
        ]


class UserList(HasPermissionsMixin, ProfileMixin, PaginatorMixin, FilterView):
    required_permission = "user_view_list"
    filterset_class = UserProfileFilterSet
    context_object_name = "users"


class UserListExport(ExportMixin, UserList):
    export_class = UserResource()
    export_filename = _("Utilisateur·rice·s")

    def get_export_class(self, request):
        """
        From ExportMixin
        Override export_class when User has no permission
        """
        if not has_permission(self.request.user, "user_export_all_fields"):
            return CollaboratorUserResource()
        return super().get_export_class(request)


class UserDetailedList(UserList):
    paginate_by = None
    context_object_name = "users"
    template_name = "auth/user_detailed_list.html"
    page_title = _("Liste des utilisateur·rice·s")

    def get_context_data(self, **kwargs):
        context = super(UserDetailedList, self).get_context_data(**kwargs)
        # Add our page title
        context["page_title"] = self.page_title
        return context


class HelpersList(UserDetailedList):
    page_title = _("Liste des moniteur·trice·s")

    def get_queryset(self):
        return (
            super(HelpersList, self)
            .get_queryset()
            .filter(profile__formation__in=FORMATION_KEYS)
        )


class ActorsList(UserDetailedList):
    page_title = _("Liste des intervenant·e·s")

    def get_queryset(self):
        return (
            super(ActorsList, self)
            .get_queryset()
            .exclude(profile__actor_for__isnull=True)
        )



from __future__ import unicode_literals

from django.contrib.auth import get_user_model
from django.db import migrations
from rolepermissions.checkers import has_role
from rolepermissions.permissions import grant_permission, revoke_permission


def add_user_can_resend_creds_to_power_user(apps, schema_editor):
    # We can't import the Person model directly as it may be a newer
    # version than this migration expects. We use the historical version.
    User = get_user_model()
    for user in User.objects.all():
        if has_role(user, "power_user"):
            grant_permission(user, "user_can_resend_credentials")
            user.save()


def remove_user_can_resend_creds_from_power_user(apps, schema_editor):
    # We can't import the Person model directly as it may be a newer
    # version than this migration expects. We use the historical version.
    User = get_user_model()
    for user in User.objects.all():
        if has_role(user, "power_user"):
            revoke_permission(user, "user_can_resend_credentials")
            user.save()


class Migration(migrations.Migration):
    dependencies = [
        ("user", "0027_auto_20170329_1231"),
    ]

    operations = [
        migrations.RunPython(
            add_user_can_resend_creds_to_power_user,
            remove_user_can_resend_creds_from_power_user,
        ),
    ]



from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("user", "0014_auto_20160901_1744"),
    ]

    operations = [
        migrations.AddField(
            model_name="userprofile",
            name="office_member",
            field=models.BooleanField(default=False, verbose_name="Bureau Défi Vélo"),
        ),
    ]



from __future__ import unicode_literals

from django.db import migrations


def migrate_addresses(apps, schema_editor):
    Address = apps.get_model("common", "Address")
    UserProfile = apps.get_model("user", "UserProfile")
    for up in UserProfile.objects.all():
        ad = Address.objects.create(
            address_street=up.address_street,
            address_no=up.address_no,
            address_zip=up.address_zip,
            address_city=up.address_city,
            address_canton=up.address_canton,
        )
        up.address_ptr = ad
        up.save()


class Migration(migrations.Migration):
    dependencies = [
        ("user", "0007_userprofile_address"),
    ]

    operations = [migrations.RunPython(migrate_addresses)]



from django.contrib.auth.management.commands import createsuperuser
from django.contrib.auth.models import User, UserManager
from django.core.exceptions import ValidationError
from django.db import transaction

from allauth.account.models import EmailAddress

from apps.user.models import UserProfile


class ProxyUserManager(UserManager):
    @transaction.atomic
    def _create_user(self, username, email, password, **extra_fields):
        normalized_email = self.normalize_email(email)
        if self.filter(email=normalized_email).exists():
            raise ValidationError("User with this email already exists.")
        user = super()._create_user(username, email, password, **extra_fields)
        UserProfile.objects.create(user=user)
        return user

    @transaction.atomic
    def create_superuser(self, username, email, password, **extra_fields):
        user = super().create_superuser(username, email, password, **extra_fields)
        email_obj, _ = EmailAddress.objects.update_or_create(
            user=user, email=user.email, defaults=dict(verified=True)
        )
        email_obj.set_as_primary()
        return user


class ProxyUser(User):
    class Meta:
        proxy = True

    objects = ProxyUserManager()


class Command(createsuperuser.Command):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.UserModel = ProxyUser



import datetime

from django.contrib.auth import get_user_model
from django.urls import reverse

from apps.challenge.tests.factories import (
    QualificationFactory,
    SeasonFactory,
    SessionFactory,
)
from apps.orga.tests.factories import OrganizationFactory
from apps.salary import HOURLY_RATE_HELPER, timesheets_overview
from apps.user.tests.factories import UserFactory
from defivelo.roles import user_cantons
from defivelo.tests.utils import (
    AuthClient,
    CollaboratorAuthClient,
    PowerUserAuthClient,
    StateManagerAuthClient,
)

from .factories import TimesheetFactory, ValidatedTimesheetFactory


def test_power_user_can_see_all_users_in_overview(db):
    client = PowerUserAuthClient()
    QualificationFactory(
        actor=UserFactory(first_name="Maurice", last_name="Moss"),
        session=SessionFactory(day=datetime.date(2019, 4, 11)),
    )

    response = client.get(reverse("salary:timesheets-overview", kwargs={"year": 2019}))

    assert "Maurice Moss" in response.content.decode()


def test_state_manager_can_only_see_managed_users(db):
    client = StateManagerAuthClient()
    managed_cantons = user_cantons(client.user)

    QualificationFactory(
        actor=UserFactory(
            first_name="Maurice",
            last_name="Moss",
            profile__affiliation_canton=managed_cantons[0],
        ),
        session=SessionFactory(
            day=datetime.date(2019, 4, 11),
            orga=OrganizationFactory(address_canton=managed_cantons[0]),
        ),
    )
    QualificationFactory(
        actor=UserFactory(
            first_name="Jen", last_name="Barber", profile__affiliation_canton="GE"
        ),
        session=SessionFactory(
            day=datetime.date(2019, 4, 20),
            orga=OrganizationFactory(address_canton="GE"),
        ),
    )

    response = client.get(reverse("salary:timesheets-overview", kwargs={"year": 2019}))

    assert "Maurice Moss" in response.content.decode()
    assert "Jen Barber" not in response.content.decode()


def test_collaborator_can_only_see_own_entries(db):
    client = CollaboratorAuthClient()
    client.user.profile.affiliation_canton = "VD"

    QualificationFactory(
        actor=UserFactory(
            first_name="Jen", last_name="Barber", profile__affiliation_canton="VD"
        ),
        session=SessionFactory(day=datetime.date(2019, 4, 11)),
    )
    QualificationFactory(
        actor=client.user,
        session=SessionFactory(day=datetime.date(2019, 4, 12)),
    )

    response = client.get(reverse("salary:timesheets-overview", kwargs={"year": 2019}))

    assert (
        reverse(
            "salary:user-timesheets",
            kwargs={"year": 2019, "month": 4, "pk": client.user.pk},
        )
        in response.content.decode()
    )
    assert "Jen Barber" not in response.content.decode()


def test_authed_user_cannot_timesheet(db):
    client = AuthClient()
    client.user.profile.affiliation_canton = "VD"

    QualificationFactory(
        actor=UserFactory(
            first_name="Mark", last_name="Brown", profile__affiliation_canton="VD"
        ),
        session=SessionFactory(day=datetime.date(2019, 4, 11)),
    )
    QualificationFactory(
        actor=client.user,
        session=SessionFactory(day=datetime.date(2019, 4, 12)),
    )

    response = client.get(reverse("salary:timesheets-overview", kwargs={"year": 2019}))
    assert response.status_code == 403  # Forbidden


def test_matrix_shows_months_with_missing_timesheets(db):
    user = UserFactory(first_name="Jen", last_name="Barber")
    QualificationFactory(
        actor=user,
        session=SessionFactory(day=datetime.date(2019, 4, 11)),
    )

    matrix = timesheets_overview.get_timesheets_status_matrix(2019, [user])

    assert user in matrix
    # Check that all months have "0" (ie. no status), except for April that has a
    # missing timesheet
    assert (
        matrix[user]
        == [0] * 3 + [timesheets_overview.TimesheetStatus.TIMESHEET_MISSING] + [0] * 8
    )


def test_matrix_shows_months_with_non_validated_timesheets(db):
    user = UserFactory(first_name="Jen", last_name="Barber")
    date = datetime.date(2019, 4, 11)
    QualificationFactory(
        actor=user,
        session=SessionFactory(day=date),
    )
    TimesheetFactory(date=date, user=user)

    matrix = timesheets_overview.get_timesheets_status_matrix(2019, [user])

    assert user in matrix
    # Check that all months have "0" except for April that has a timesheet awaiting
    # validation
    assert (
        matrix[user]
        == [0] * 3
        + [timesheets_overview.TimesheetStatus.TIMESHEET_NOT_VALIDATED]
        + [0] * 8
    )


def test_matrix_shows_months_with_validated_timesheets(db):
    user = UserFactory(first_name="Jen", last_name="Barber")
    date = datetime.date(2019, 4, 11)
    QualificationFactory(
        actor=user,
        session=SessionFactory(day=date),
    )
    ValidatedTimesheetFactory(
        date=date,
        user=user,
    )

    matrix = timesheets_overview.get_timesheets_status_matrix(2019, [user])

    assert user in matrix
    # Check that all months have "0" except for April that has a timesheet that is validated
    assert (
        matrix[user]
        == [0] * 3 + [timesheets_overview.TimesheetStatus.TIMESHEET_VALIDATED] + [0] * 8
    )


def test_matrix_filters_on_selected_canton(db):
    user = UserFactory(first_name="Jen", last_name="Barber")
    vd_date = datetime.date(2019, 4, 11)
    ge_date = datetime.date(2019, 5, 11)

    SeasonFactory(cantons=["VD"], year=2019, month_start=1, n_months=5)
    SeasonFactory(cantons=["GE"], year=2019, month_start=1, n_months=5)
    QualificationFactory(
        actor=user,
        session=SessionFactory(
            day=vd_date, orga=OrganizationFactory(address_canton="VD")
        ),
    )
    QualificationFactory(
        actor=user,
        session=SessionFactory(
            day=ge_date, orga=OrganizationFactory(address_canton="GE")
        ),
    )
    TimesheetFactory(date=vd_date, user=user)
    TimesheetFactory(date=ge_date, user=user)

    # Filter for VD
    matrix = timesheets_overview.get_timesheets_status_matrix(
        2019, [user], cantons=["VD"]
    )

    assert (
        matrix[user][3] == timesheets_overview.TimesheetStatus.TIMESHEET_NOT_VALIDATED
    )
    assert matrix[user][4] == 0

    # No fitler
    matrix = timesheets_overview.get_timesheets_status_matrix(
        2019, [user], cantons=None
    )
    assert (
        matrix[user][3] == timesheets_overview.TimesheetStatus.TIMESHEET_NOT_VALIDATED
    )
    assert (
        matrix[user][4] == timesheets_overview.TimesheetStatus.TIMESHEET_NOT_VALIDATED
    )


def test_orphaned_timesheets_in_a_month(db):
    user = UserFactory(
        first_name="Jen", last_name="Barber", profile__affiliation_canton="VD"
    )
    date = datetime.date(2019, 4, 11)
    ts = ValidatedTimesheetFactory(
        date=date,
        user=user,
    )

    orphaned_timesheets = timesheets_overview.get_orphaned_timesheets_per_month(
        date.year,
        users=get_user_model().objects.all(),
        month=date.month,
        cantons=["VD"],
    )

    # Check that our timesheet without a matching Qualification is in the orphaned set
    assert ts in orphaned_timesheets


def test_non_orphaned_timesheets_in_a_month(db):
    user = UserFactory(
        first_name="Jen", last_name="Barber", profile__affiliation_canton="VD"
    )
    date = datetime.date(2019, 4, 11)
    ts = ValidatedTimesheetFactory(
        date=date,
        user=user,
    )
    QualificationFactory(
        actor=user,
        session=SessionFactory(day=date),
    )

    orphaned_timesheets = timesheets_overview.get_orphaned_timesheets_per_month(
        date.year,
        users=get_user_model().objects.all(),
        month=date.month,
        cantons=["VD"],
    )

    # Check that our timesheet with a matching Qualification is not in the orphaned set
    assert ts not in orphaned_timesheets


def test_matrix_shows_timesheets_total(db):
    user = UserFactory(first_name="Jen", last_name="Barber")
    date = datetime.date(2019, 4, 11)
    QualificationFactory(
        actor=user,
        session=SessionFactory(day=date),
    )
    TimesheetFactory(date=date, user=user, overtime=5)

    amounts = timesheets_overview.get_timesheets_amount_by_month(2019, [user])

    # Check that all months have "0" except for April that has a timesheet that is validated
    assert amounts == [0] * 3 + [5 * HOURLY_RATE_HELPER] + [0] * 8


def test_timesheets_total_filters_on_selected_canton(db):
    user = UserFactory(first_name="Jen", last_name="Barber")
    vd_date = datetime.date(2019, 4, 11)
    ge_date = datetime.date(2019, 5, 11)

    # Create sessions in different cantons
    vd_orga = OrganizationFactory(address_canton="VD")
    ge_orga = OrganizationFactory(address_canton="GE")

    QualificationFactory(
        actor=user,
        session=SessionFactory(day=vd_date, orga=vd_orga),
    )
    QualificationFactory(
        actor=user,
        session=SessionFactory(day=ge_date, orga=ge_orga),
    )

    # Create timesheets for both dates
    TimesheetFactory(date=vd_date, user=user, overtime=5)
    TimesheetFactory(date=ge_date, user=user, overtime=3)

    # Filter for VD canton only
    amounts = timesheets_overview.get_timesheets_amount_by_month(
        2019, [user], cantons=["VD"]
    )
    # Only the VD timesheet (5 hours in April) should be counted
    assert amounts == [0] * 3 + [5 * HOURLY_RATE_HELPER] + [0] * 8

    # Filter for GE canton only
    amounts = timesheets_overview.get_timesheets_amount_by_month(
        2019, [user], cantons=["GE"]
    )
    # Only the GE timesheet (3 hours in May) should be counted
    assert amounts == [0] * 4 + [3 * HOURLY_RATE_HELPER] + [0] * 7

    # No filter - should include both
    amounts = timesheets_overview.get_timesheets_amount_by_month(
        2019, [user], cantons=None
    )
    # Both timesheets should be counted
    assert (
        amounts
        == [0] * 3 + [5 * HOURLY_RATE_HELPER] + [3 * HOURLY_RATE_HELPER] + [0] * 7
    )


def test_state_manager_sees_reminder_button_for_month_with_missing_timesheets(db):
    client = StateManagerAuthClient()
    managed_cantons = user_cantons(client.user)

    helper = UserFactory(
        first_name="Maurice",
        last_name="Moss",
        profile__affiliation_canton=managed_cantons[0],
    )

    date_validated = datetime.date(2019, 2, 11)
    orga = OrganizationFactory(address_canton=managed_cantons[0])
    QualificationFactory(
        actor=helper,
        session=SessionFactory(day=date_validated, orga=orga),
    )
    TimesheetFactory(
        date=date_validated,
        user=helper,
    )

    date_missing = datetime.date(2019, 3, 11)
    QualificationFactory(
        actor=helper,
        session=SessionFactory(day=date_missing, orga=orga),
    )

    response = client.get(reverse("salary:timesheets-overview", kwargs={"year": 2019}))
    assert response.status_code == 200
    show_jan, show_feb, show_march, *_ = response.context["show_reminder_button_months"]
    assert not show_jan and not show_feb and show_march
    assert "Envoyer un rappel aux collabora" in response.content.decode()


def test_collaborator_doesnt_see_reminder_button(db):
    client = CollaboratorAuthClient()
    QualificationFactory(
        actor=client.user,
        session=SessionFactory(day=datetime.date(2019, 3, 11)),
    )
    response = client.get(reverse("salary:timesheets-overview", kwargs={"year": 2019}))
    assert response.status_code == 200
    assert "Envoyer un rappel aux collabora" not in response.content.decode()


def test_state_manager_sees_orphanage_fix_button_for_month_with_orphaned_timesheets(db):
    client = StateManagerAuthClient()
    managed_cantons = user_cantons(client.user)

    helper = UserFactory(
        first_name="Maurice",
        last_name="Moss",
        profile__affiliation_canton=managed_cantons[0],
    )
    helper.profile.save()

    date_ts = datetime.date(2019, 4, 11)
    ts = TimesheetFactory(
        date=date_ts,
        user=helper,
    )
    # Create a Qualification, but at a different date
    date_quali = datetime.date(2019, 4, 15)
    QualificationFactory(
        actor=helper,
        session=SessionFactory(day=date_quali, orga__address_canton=managed_cantons[0]),
    )

    response = client.get(reverse("salary:timesheets-overview", kwargs={"year": 2019}))

    assert response.status_code == 200
    assert response.context["orphaned_timesheets"][1] == set()
    assert response.context["orphaned_timesheets"][2] == set()
    assert response.context["orphaned_timesheets"][3] == set()
    assert ts in response.context["orphaned_timesheets"][4]

    assert "1 entrée orpheline" in response.content.decode()



# Generated by Django 1.11.20 on 2019-02-25 15:31
from __future__ import unicode_literals

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("challenge", "0051_auto_20181019_1411"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalhelperseasonworkwish",
            name="helper",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to=settings.AUTH_USER_MODEL,
                verbose_name="Moniteur",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhelperseasonworkwish",
            name="season",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="challenge.Season",
                verbose_name="Saison",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhelpersessionavailability",
            name="helper",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to=settings.AUTH_USER_MODEL,
                verbose_name="Moniteur",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhelpersessionavailability",
            name="session",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="challenge.Session",
                verbose_name="Session",
            ),
        ),
        migrations.AlterField(
            model_name="historicalqualification",
            name="activity_A",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="challenge.QualificationActivity",
                verbose_name="Agilité",
            ),
        ),
        migrations.AlterField(
            model_name="historicalqualification",
            name="activity_B",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="challenge.QualificationActivity",
                verbose_name="Mécanique",
            ),
        ),
        migrations.AlterField(
            model_name="historicalqualification",
            name="activity_C",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="challenge.QualificationActivity",
                verbose_name="Rencontre",
            ),
        ),
        migrations.AlterField(
            model_name="historicalqualification",
            name="actor",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to=settings.AUTH_USER_MODEL,
                verbose_name="Intervenant",
            ),
        ),
        migrations.AlterField(
            model_name="historicalqualification",
            name="leader",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to=settings.AUTH_USER_MODEL,
                verbose_name="Moniteur 2",
            ),
        ),
        migrations.AlterField(
            model_name="historicalseason",
            name="leader",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to=settings.AUTH_USER_MODEL,
                verbose_name="Chargé de projet",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsession",
            name="address_ptr",
            field=models.ForeignKey(
                auto_created=True,
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                parent_link=True,
                related_name="+",
                to="common.Address",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsession",
            name="orga",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="orga.Organization",
                verbose_name="Établissement",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsession",
            name="superleader",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to=settings.AUTH_USER_MODEL,
                verbose_name="Moniteur + / Photographe",
            ),
        ),
    ]



import datetime

from django.urls import reverse

from apps.salary.models import Timesheet
from apps.salary.tests.factories import TimesheetFactory
from apps.user.tests.factories import UserFactory
from defivelo.tests.utils import PowerUserAuthClient

from ...user import FORMATION_M2
from .. import CHOSEN_AS_ACTOR
from ..models import HelperSessionAvailability
from .factories import (
    QualificationActivityFactory,
    QualificationFactory,
    SessionFactory,
)
from .test_seasons import SeasonTestCaseMixin


class TimesheetsTestCase(SeasonTestCaseMixin):
    def setUp(self):
        self.client = PowerUserAuthClient()
        super(TimesheetsTestCase, self).setUp()

    def test_related_timesheets_are_displayed_on_quali_edition(self):
        user = self.client.user
        session1 = self.sessions[0]
        quali = QualificationFactory(actor=user, session=session1)
        session2 = SessionFactory(
            orga=session1.orga, day=session1.day + datetime.timedelta(weeks=5)
        )
        QualificationFactory(actor=user, session=session2)

        timesheet1 = TimesheetFactory(user=user, date=self.sessions[0].day)
        timesheet2 = TimesheetFactory(user=user, date=session2.day)

        response = self.client.get(
            reverse(
                "quali-update",
                kwargs={
                    "pk": quali.pk,
                    "seasonpk": self.season.pk,
                    "sessionpk": session1.pk,
                },
            )
        )
        content = response.rendered_content

        # Related timesheet
        assert (
            reverse(
                "salary:user-timesheets",
                kwargs={
                    "year": timesheet1.date.year,
                    "month": timesheet1.date.month,
                    "pk": user.pk,
                },
            )
            in content
        )

        # Non related timesheet
        assert (
            reverse(
                "salary:user-timesheets",
                kwargs={
                    "year": timesheet2.date.year,
                    "month": timesheet2.date.month,
                    "pk": user.pk,
                },
            )
            not in content
        )

    def test_timesheets_are_displayed_on_quali_deletion(self):
        user = self.client.user
        session1 = self.sessions[0]
        quali = QualificationFactory(actor=user, session=session1)
        session2 = SessionFactory(
            orga=session1.orga, day=session1.day + datetime.timedelta(weeks=5)
        )
        QualificationFactory(actor=user, session=session2)

        timesheet1 = TimesheetFactory(user=user, date=session1.day)
        timesheet2 = TimesheetFactory(user=user, date=session2.day)

        response = self.client.get(
            reverse(
                "quali-delete",
                kwargs={
                    "pk": quali.pk,
                    "seasonpk": self.season.pk,
                    "sessionpk": session1.pk,
                },
            )
        )

        content = response.rendered_content

        # Related timesheet
        assert (
            reverse(
                "salary:user-timesheets",
                kwargs={
                    "year": timesheet1.date.year,
                    "month": timesheet1.date.month,
                    "pk": user.pk,
                },
            )
            in content
        )

        # Non related timesheet
        assert (
            reverse(
                "salary:user-timesheets",
                kwargs={
                    "year": timesheet2.date.year,
                    "month": timesheet2.date.month,
                    "pk": user.pk,
                },
            )
            not in content
        )

    def test_day_is_enabled_on_session_edition_if_no_timesheet_exist(self):
        response = self.client.get(
            reverse(
                "session-update",
                kwargs={"pk": self.sessions[0].pk, "seasonpk": self.season.pk},
            )
        )
        assert (
            not response.context["form"]
            .fields["day"]
            .widget.attrs.get("readonly", False)
        )

    def test_day_is_disabled_on_session_edition_if_a_timesheet_exists(self):
        user = self.client.user
        QualificationFactory(actor=user, session=self.sessions[0])
        TimesheetFactory(user=user, date=self.sessions[0].day)

        response = self.client.get(
            reverse(
                "session-update",
                kwargs={"pk": self.sessions[0].pk, "seasonpk": self.season.pk},
            )
        )
        assert (
            response.context["form"].fields["day"].widget.attrs.get("readonly", False)
        )

    def test_specific_timesheets_are_deleted_on_quali_edition(self):
        user1 = self.client.user
        user1_activity = QualificationActivityFactory()
        user1.profile.actor_for.add(user1_activity)
        user2 = UserFactory()
        session = self.sessions[0]

        HelperSessionAvailability.objects.create(
            session=session, helper=user1, availability="y", chosen_as=CHOSEN_AS_ACTOR
        )

        quali = QualificationFactory(actor=user1, leader=user2, session=session)

        timesheet1_pk = TimesheetFactory(user=user1, date=session.day).pk
        timesheet2_pk = TimesheetFactory(user=user2, date=session.day).pk

        self.client.post(
            reverse(
                "quali-update",
                kwargs={
                    "pk": quali.pk,
                    "seasonpk": self.season.pk,
                    "sessionpk": session.pk,
                },
            ),
            data={
                "session": session.pk,
                "name": quali.name,
                "class_teacher_fullname": "Nina Strahm",
                "class_teacher_natel": "+41319242400",
                "n_participants": "25",
                "n_bikes": "",
                "n_helmets": "0",
                "leader": "",
                "helpers[]": "",
                "activity_A": "2",
                "activity_B": "4",
                "activity_C": user1_activity.pk,
                "actor": user1.pk,
                "comments": "The leader has been removed",
            },
        )

        assert Timesheet.objects.filter(pk=timesheet1_pk).exists()
        assert not Timesheet.objects.filter(pk=timesheet2_pk).exists()

    def test_timesheets_are_deleted_on_quali_deletion(self):
        user = self.client.user
        session = self.sessions[0]
        quali = QualificationFactory(actor=user, session=session)
        timesheet1_pk = TimesheetFactory(user=user, date=session.day).pk
        timesheet2_pk = TimesheetFactory(
            user=user, date=session.day + datetime.timedelta(days=1)
        ).pk

        self.client.post(
            reverse(
                "quali-delete",
                kwargs={
                    "pk": quali.pk,
                    "seasonpk": self.season.pk,
                    "sessionpk": session.pk,
                },
            )
        )

        assert not Timesheet.objects.filter(pk=timesheet1_pk).exists()
        assert Timesheet.objects.filter(pk=timesheet2_pk).exists()

    def test_timesheets_are_deleted_on_session_deletion(self):
        user = self.client.user
        session = self.sessions[0]
        QualificationFactory(actor=user, session=session)
        timesheet1_pk = TimesheetFactory(user=user, date=session.day).pk
        timesheet2_pk = TimesheetFactory(
            user=user, date=session.day + datetime.timedelta(days=1)
        ).pk

        response = self.client.post(
            reverse(
                "session-delete",
                kwargs={
                    "pk": session.pk,
                    "seasonpk": self.season.pk,
                },
            )
        )
        assert response.status_code == 302

        assert not Timesheet.objects.filter(pk=timesheet1_pk).exists()
        assert Timesheet.objects.filter(pk=timesheet2_pk).exists()

    def test_quali_edition_displays_distinct_timesheets(self):
        actor = self.client.user
        actor.profile.actor_for.set([QualificationActivityFactory()])
        actor.profile.formation = FORMATION_M2
        actor.profile.save()

        session1 = self.sessions[0]
        session2 = SessionFactory(orga__address_canton="VD", day=session1.day)

        QualificationFactory.create_batch(2, session=session2, helpers=[actor])

        qualifications = session1.qualifications.all()
        qualification = qualifications[0]

        HelperSessionAvailability.objects.get_or_create(
            session=session1,
            helper=actor,
            availability="y",
            chosen_as=CHOSEN_AS_ACTOR,
        )
        qualification.actor = actor
        qualification.save()

        TimesheetFactory(user=actor, date=session1.day)

        response = self.client.get(
            reverse(
                "quali-update",
                kwargs={
                    "pk": qualifications[0].pk,
                    "seasonpk": self.season.pk,
                    "sessionpk": session1.pk,
                },
            )
        )

        timesheets = response.context["timesheets"]

        assert len(timesheets) == 1



# Generated by Django 4.2.6 on 2025-11-13 13:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('challenge', '0086_inclusive_language'),
        ('user', '0075_phone_length_fix'),
    ]

    operations = [
        migrations.AlterField(
            model_name='userprofile',
            name='actor_for',
            field=models.ManyToManyField(blank=True, limit_choices_to={'category': 'C'}, related_name='actor_for', to='challenge.qualificationactivity', verbose_name='Intervenant·e'),
        ),
        migrations.AlterField(
            model_name='userprofile',
            name='formation',
            field=models.CharField(blank=True, choices=[('', '----------'), ('M1', 'Moniteur·trice 1'), ('M2', 'Moniteur·trice 2')], max_length=2, verbose_name='Formation'),
        ),
        migrations.AlterField(
            model_name='userprofile',
            name='status',
            field=models.PositiveSmallIntegerField(choices=[(0, '---------'), (10, 'Actif·ve'), (20, 'Réserve'), (30, 'Inactif·ve'), (40, 'Archivé·e'), (99, 'Supprimé·e')], default=10, verbose_name='Statut'),
        ),
    ]



from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("challenge", "0026_auto_20160926_1655"),
    ]

    operations = [
        migrations.AlterField(
            model_name="season",
            name="leader",
            field=models.ForeignKey(
                verbose_name="Chargé de projet",
                default=3,
                to=settings.AUTH_USER_MODEL,
                on_delete=models.CASCADE,
            ),
            preserve_default=False,
        ),
    ]



# defivelo-intranet -- Outil métier pour la gestion du Défi Vélo
# Copyright (C) 2016 Didier Raboud <me+defivelo@odyx.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.contrib.auth import get_user_model

import factory
from factory import Faker
from factory.django import DjangoModelFactory

from apps.user import FORMATION_M1
from apps.user.models import USERSTATUS_ACTIVE, UserProfile, get_new_username


class UserProfileFactory(DjangoModelFactory):
    class Meta:
        model = UserProfile

    formation = FORMATION_M1
    status = USERSTATUS_ACTIVE


class UserFactory(DjangoModelFactory):
    class Meta:
        model = get_user_model()

    profile = factory.RelatedFactory(UserProfileFactory, "user")
    username = factory.LazyFunction(get_new_username)
    first_name = Faker("first_name")
    last_name = Faker("last_name")
    email = factory.Sequence(lambda n: "user%d@example.com" % n)
    # Normal users can't login, and don't have passwords
    password = factory.PostGenerationMethodCall("set_unusable_password")
    is_active = False



from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("common", "0003_address_address_additional"),
    ]

    operations = [
        migrations.AlterField(
            model_name="address",
            name="address_canton",
            field=models.CharField(blank=True, verbose_name="Canton", max_length=5),
        ),
    ]



#!/usr/bin/env python
# defivelo-intranet -- Outil métier pour la gestion du Défi Vélo
# Copyright (C) 2015 Didier Raboud <me+defivelo@odyx.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import os
import sys

from defivelo import get_project_root_path, import_env_vars

if __name__ == "__main__":
    if 'test' in sys.argv:
        env_dir = os.path.join('envdir', 'tests')
    else:
        env_dir = 'envdir'

    import_env_vars(os.path.join(get_project_root_path(), env_dir))

    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "defivelo.settings.dev")

    from django.core.management import execute_from_command_line

    execute_from_command_line(sys.argv)



# defivelo-intranet -- Outil métier pour la gestion du Défi Vélo
# Copyright (C) 2015 Didier Raboud <me+defivelo@odyx.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.utils.translation import gettext_lazy as _

AVAILABILITY_FIELDKEY_HELPER_PREFIX = "avail-h{hpk}-"
STAFF_FIELDKEY_HELPER_PREFIX = "staff-h{hpk}-"
SUPERLEADER_FIELDKEY_HELPER_PREFIX = "superleader-h{hpk}-"
CHOICE_FIELDKEY_HELPER_PREFIX = "choice-h{hpk}-"
CONFLICT_FIELDKEY_HELPER_PREFIX = "conflict-h{hpk}-"
SEASON_WORKWISH_FIELDKEY = "season-ww-h{hpk}"

AVAILABILITY_FIELDKEY = AVAILABILITY_FIELDKEY_HELPER_PREFIX + "s{spk}"
STAFF_FIELDKEY = STAFF_FIELDKEY_HELPER_PREFIX + "s{spk}"
SUPERLEADER_FIELDKEY = SUPERLEADER_FIELDKEY_HELPER_PREFIX + "s{spk}"
CHOICE_FIELDKEY = CHOICE_FIELDKEY_HELPER_PREFIX + "s{spk}"
CONFLICT_FIELDKEY = CONFLICT_FIELDKEY_HELPER_PREFIX + "s{spk}"

MAX_MONO1_PER_QUALI = 2

CHOSEN_AS_NOT = 0
CHOSEN_AS_LEGACY = 1
CHOSEN_AS_ACTOR = 2
CHOSEN_AS_HELPER = 3
CHOSEN_AS_LEADER = 4
CHOSEN_AS_REPLACEMENT = 5
CHOICE_CHOICES = (
    (CHOSEN_AS_NOT, _("Pas choisi")),
    (CHOSEN_AS_LEGACY, _("Choisi")),  # À ne pas réutiliser
    (CHOSEN_AS_ACTOR, _("Comme intervenant·e")),
    (CHOSEN_AS_REPLACEMENT, _("Moniteur·trice de secours")),
    (CHOSEN_AS_HELPER, _("Moniteur·trice 1")),
    (CHOSEN_AS_LEADER, _("Moniteur·trice 2")),
)

CHOSEN_KEYS = [c[0] for c in CHOICE_CHOICES if c[0] != CHOSEN_AS_NOT]



# Generated by Django 1.11.4 on 2017-08-31 12:37
from __future__ import unicode_literals

from django.contrib.auth import get_user_model
from django.db import migrations
from rolepermissions.checkers import has_role
from rolepermissions.roles import assign_role, clear_roles


def add_user_set_role_to_power_user(apps, schema_editor):
    # We can't import the Person model directly as it may be a newer
    # version than this migration expects. We use the historical version.
    User = get_user_model()
    for user in User.objects.all():
        if has_role(user, "power_user"):
            clear_roles(user)
            assign_role(user, "power_user")

        if has_role(user, "state_manager"):
            clear_roles(user)
            assign_role(user, "state_manager")


def remove_user_set_role_from_power_user(apps, schema_editor):
    # We can't import the Person model directly as it may be a newer
    # version than this migration expects. We use the historical version.
    pass


class Migration(migrations.Migration):
    dependencies = [
        ("user", "0059_userprofile_phone"),
    ]

    operations = [
        migrations.RunPython(
            add_user_set_role_to_power_user, remove_user_set_role_from_power_user
        ),
    ]



from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("common", "0001_initial"),
        ("user", "0006_userprofile_actor_for"),
    ]

    operations = [
        migrations.AddField(
            model_name="userprofile",
            name="address_ptr",
            field=models.ForeignKey(
                to="common.Address", null=True, on_delete=models.CASCADE
            ),
        ),
    ]
